#ifndef ANIMATION_H
#define ANIMATION_H

#include "XMLResource.h"
#pragma once
//! Animation class
/*!
	This class provides an interface for derived animation classes to use
	as well as loading/saving it in XML.
*/
class CAnimation : public CXMLResource
{
public:
	
	CAnimation();
	virtual ~CAnimation();
	/*!
		Method details:
		@param fTimePerFrame - Amount of time for the frame to wait before advancing to next
		\return 1 If animation is complete, 0 otherwise
	*/
	virtual int Animate(double fTimePerFrame);
	/*!
		Method details:
		@param fPauseTime - Pause for infinite amount of time or one specified by user
		\return 1 If animation is paused, 0 otherwise
	*/
	virtual int Pause(double fPauseTime = INFINITE);
	/*!
		Method details:
		@param fTimeToWait - Time to wait before resuming the animation
		\return 1 If it is still waiting, 0 otherwise
	*/
	virtual int Resume(double fTimeToWait = 0);
	/*!
		Method details:
		\return 1 If animation has been stopped, 0 otherwise
	*/
	virtual int Stop();
	/*!
		Method details:
		Clean-up method
	*/
	virtual void Destroy();
	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that holds data for the animation
		@param strFilename - String containing filename where the data for the animation is stored
	*/
	virtual int LoadXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that will be the parent hold the data for this animation
		@param strFilename - String containing filename where the data for the animation is stored
	*/
	virtual int SaveXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		This is an implementation of Custom RTTI
	*/
	virtual bool IsOfType(eEntityType eType);

};

#endif
