#ifndef ANIMATIONMANAGER_H
#define ANIMATIONMANAGER_H

#pragma once

#include "ResourceManager.h"
#include "TextureFrameAnimation.h"

//!Animation Manager class
/*!
	This class provides means to managing various animations.
	It manages a collection of animations (although not pooled yet!) and allow to retrieve
	them by index, name, ID or filename.
*/
class CAnimationManager : public CResourceManager, public CSingleton<CAnimationManager>
{
public:
	CAnimationManager();
	virtual ~CAnimationManager();

public:
	/*!
		Method details:
		@param pDataNode - Pointer to XML Data node containing data for the animation
		@param strFilename - Name of the file containing data for the animation
		@param uiID - ID for the animation to be loaded
		@param strName - Name for the animation to be loaded
		\return  Pointer to the animation if it has been loaded succesfully, NULL if not
	*/
	virtual CAnimation *LoadResource(TiXmlNode *pDataNode, CString strFilename, UINT uiID = 0, CString strName = "");
	/*!
		Method details:
		@param uiIndex - Index by which to retrieve the animation
		@param uiID - ID by which to retrieve the animation
		@param strName - Name by which to retrieve the animation
		@param strFilename - Filename by which to retrieve the animation
		\return Pointer to the animation if it has been found using given parameters, NULL if not
	*/
	virtual CAnimation *GetResource(int uiIndex, int uiID = -1, CString strName = "", CString strFilename = "");
	/*!
		Method details:
		@param strName - Name of the animation to add
		@param uiID - ID of the animation to add
		@param ResourceType - Type of the animation to add
		\return Pointer to the animation if it has been added succesfully using given parameters, NULL if not
	*/
	virtual CAnimation *AddResource(CString strName, UINT uiID = 0, eEntityType ResourceType = Entity_Default);
	/*!
		Method details:
		@param pResource - Pointer to the animation to remove
		\return 1 if it has been found and removed succesfully, 0 otherwise
	*/
	int RemoveResource(CAnimation *pAnimation);

	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that holds data for this texture
		@param strFilename - String containing filename where the data for the animation is stored
	*/
	virtual int LoadXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that will be the parent hold the data for this texture
		@param strFilename - String containing filename where the data for the animation is stored
	*/
	virtual int SaveXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		This is where clean-up occurs
	*/
	virtual void Destroy();
	/*!
		Method details:
		This is an implementation of Custom RTTI
	*/
	virtual bool IsOfType(eEntityType eType);
};

#endif