#include "Application.h"

CApplication::CApplication()
{
	CGlobalLogger *pGlobalLogger = new CGlobalLogger("GlobalLog.txt");
	
	time_t RawTime;
	struct tm * TimeInfo;

	time ( &RawTime );
	TimeInfo = localtime ( &RawTime );

	CGlobalLogger::GetSingleton().Write("\t\t\t\t\t Log for %s \n", asctime (TimeInfo));
	
	CGlobalLogger::GetSingleton().Write("Application - Instantiated\n");
	CAnimationManager *pAnimationManager = new CAnimationManager;
	CFontManager *pFontManager = new CFontManager;
	CTextureManager *pTextureManager = new CTextureManager;
	CMaterialManager *pMaterialManager = new CMaterialManager;
	CInputEngine *pInputEngine = new CInputEngine;
	
	pInputEngine->Create();
	
}

CApplication::~CApplication()
{
	Destroy();

	CGlobalLogger::GetSingleton().Write("Application - Shutting down\n");
	delete CGlobalLogger::GetSingletonPtr();
}

int CApplication::Create()
{
	CTextureManager::GetSingleton().LoadXML(NULL, "texture_database.xml");
	CMaterialManager::GetSingleton().LoadXML(NULL, "material_database.xml");
	CAnimationManager::GetSingleton().LoadXML(NULL, "animation_database.xml");
	CFontManager::GetSingleton().LoadXML(NULL, "font_database.xml");

	m_pGameGUI = new CGUI;
	m_pGameGUI->Create();
	int Ret = m_pGameGUI->LoadXML(NULL, "gui.xml");

	return 1;
}

int CApplication::Run()
{
	m_pGameGUI->Update();
	m_pGameGUI->Draw();

	CInputEngine::GetSingleton().Poll();
	return 1;
}

int CApplication::Destroy()
{

	SAFEDEL(m_pGameGUI)
	delete CTextureManager::GetSingletonPtr();
	delete CMaterialManager::GetSingletonPtr();
	delete CInputEngine::GetSingletonPtr();
	delete CAnimationManager::GetSingletonPtr();
	delete CFontManager::GetSingletonPtr();

	CGlobalLogger::GetSingleton().Write("Application - Cleaning Up\n");
	
	return 1;
}