#include "StdAfx.h"
#include "BitmapFont.h"

CBitmapFont::CBitmapFont()
{
	SetEntityType(Entity_BitmapFont);

	m_uiNI = 
	m_uiCharLength = 
	m_uiCharLength = 
	m_uiQuadLength = 
	m_uiQuadLength = 
	m_uiSpacing =
	m_dwFlags = 0;

	m_pTexture = NULL;
	m_pMaterial = NULL;

	m_eFontType = Bitmap_Font;
}

CBitmapFont::~CBitmapFont()
{

}

int CBitmapFont::Create(UINT uiNormalOrItalic, UINT uiCharLength, UINT uiCharHeight, UINT uiQuadLength, UINT uiQuadHeight, UINT uiSpacing)
{
	if(m_pTexture == NULL)
		return 0;

	m_uiQuadLength = uiQuadLength;
	m_uiQuadLength = uiQuadHeight;
	m_uiCharLength = uiCharLength;
	m_uiCharLength = uiCharHeight;
	m_uiSpacing = uiSpacing;
	m_uiNI = uiNormalOrItalic;

	m_uiNumLists = 256;
	
	glPushMatrix();
	float	cx;												// Holds Our X Character Coord
	float	cy;												// Holds Our Y Character Coord
	float	quad_x = (float)m_uiQuadLength, 
			quad_y = (float)m_uiQuadLength;
	int lx = m_uiCharLength, ly = m_uiCharLength;

	m_uiFontBase = glGenLists(m_uiNumLists);				// Creating 256 Display Lists
	// glBindTexture(GL_TEXTURE_2D, texture[0]);			// Select Our Font Texture
	for (int loop=0; loop<m_uiNumLists; loop++)				// Loop Through All 256 Lists
	{
		cx = float(loop%lx)*0.0625f;						// X Position Of Current Character
		cy = float(loop/ly)*0.0625f;						// Y Position Of Current Character

		glNewList(m_uiFontBase+loop, GL_COMPILE);			// Start Building A List
		m_pTexture->Bind();
			glBegin(GL_QUADS);								// Use A Quad For Each Character
				glTexCoord2f(cx, 1-cy-0.0625f);				// Texture Coord (Bottom Left)
				glVertex2i(0, 0);							// Vertex Coord (Bottom Left)
				glTexCoord2f(cx+0.0625f, 1-cy-0.0625f);		// Texture Coord (Bottom Right)
				glVertex2i(quad_x, 0);						// Vertex Coord (Bottom Right)
				glTexCoord2f(cx+0.0625f, 1-cy);				// Texture Coord (Top Right)
				glVertex2i(quad_x, quad_y);					// Vertex Coord (Top Right)
				glTexCoord2f(cx, 1-cy);						// Texture Coord (Top Left)
				glVertex2i(0, quad_y);						// Vertex Coord (Top Left)
			glEnd();										// Done Building Our Quad (Character)
		glTranslated(m_uiQuadLength + m_uiSpacing, 0, 0);	// Move To The Right Of The Character
		glEndList();										// Done Building The Display List
	}														// Loop Until All 256 Are Built
	glPopMatrix();

	return 1;
}

int CBitmapFont::LoadXML(TiXmlNode *pDataNode, CString strFilename)
{
	TiXmlElement* pXMLElement = NULL;
	const char *pcValue = NULL;

	int iRetValue = CCustomFont::LoadXML(pDataNode, strFilename);	// LoadXML the data of derivative
	if(iRetValue !=1)
		return iRetValue;

	pXMLElement = m_pXMLElement;

	if(stricmp(pXMLElement->Value(), "Font") !=0)
		return 0;

	int Color[3] = {0, 0, 0};
	pcValue = pXMLElement->Attribute("Type");
	if(pcValue !=NULL)
	{
		if(stricmp(pcValue, "bitmap") !=0)
			return -1;								// Not bitmap font!

		pcValue = NULL;
	}

	pcValue = pXMLElement->Attribute("QuadLength");
	if(pcValue !=NULL)
	{
		m_uiQuadLength = atoi(pcValue);
		pcValue = NULL;
	}

	pcValue = pXMLElement->Attribute("QuadHeight");
	if(pcValue !=NULL)
	{
		m_uiQuadHeight = atoi(pcValue);
		pcValue = NULL;
	}

	pcValue = pXMLElement->Attribute("CharLength");
	if(pcValue !=NULL)
	{
		m_uiCharLength = atoi(pcValue);
		pcValue = NULL;
	}

	pcValue = pXMLElement->Attribute("CharHeight");
	if(pcValue !=NULL)
	{
		m_uiCharHeight = atoi(pcValue);
		pcValue = NULL;
	}

	pcValue = pXMLElement->Attribute("NI");
	if(pcValue !=NULL)
	{
		m_uiNI = atoi(pcValue);
		pcValue = NULL;
	}

	pcValue = pXMLElement->Attribute("Spacing");
	if(pcValue !=NULL)
	{
		m_uiSpacing = atoi(pcValue);
		pcValue = NULL;
	}

	m_pTexture = (CTexture *)CTextureManager::GetSingleton().LoadResource(pXMLElement, "");
	m_pMaterial = (CMaterial *)CMaterialManager::GetSingleton().LoadResource(pXMLElement, "");

	return Create(m_uiNI, m_uiCharLength, m_uiCharHeight, m_uiQuadLength, m_uiQuadHeight, m_uiSpacing);
}

int CBitmapFont::SaveXML(TiXmlNode *pDataNode, CString strFilename)
{
	TiXmlElement *pSaveElement = NULL;
	TiXmlDocument Doc;

	if(pDataNode !=NULL)
		pSaveElement = pDataNode->ToElement();
	else if(m_pXMLElement !=NULL)
		pSaveElement = m_pXMLElement;
	else
		pSaveElement = new TiXmlElement("Font");

	char pcValue[256] = "";


	if(m_eFontType == Bitmap_Font)
		pSaveElement->SetAttribute("Type", "Bitmap");

	sprintf(pcValue, "%d", m_uiQuadLength);
	pSaveElement->SetAttribute("uiQuadLength", pcValue);
	memset(pcValue, 0, sizeof(pcValue));

	sprintf(pcValue, "%d", m_uiQuadHeight);
	pSaveElement->SetAttribute("uiQuadHeight", pcValue);
	memset(pcValue, 0, sizeof(pcValue));

	sprintf(pcValue, "%d", m_uiCharLength);
	pSaveElement->SetAttribute("uiCharLength", pcValue);
	memset(pcValue, 0, sizeof(pcValue));

	sprintf(pcValue, "%d", m_uiCharHeight);
	pSaveElement->SetAttribute("uiCharHeight", pcValue);
	memset(pcValue, 0, sizeof(pcValue));

	sprintf(pcValue, "%d", m_uiNI);
	pSaveElement->SetAttribute("NI", pcValue);
	memset(pcValue, 0, sizeof(pcValue));

	sprintf(pcValue, "%d", m_uiSpacing);
	pSaveElement->SetAttribute("uiSpacing", pcValue);
	memset(pcValue, 0, sizeof(pcValue));

	if(m_pTexture !=NULL)
		pSaveElement->SetAttribute("TexByName", m_pTexture->GetName());

	if(m_pMaterial !=NULL)
		pSaveElement->SetAttribute("MatByName", m_pMaterial->GetName());

	if(GetName() !="")
		pSaveElement->SetAttribute("Name", GetName().GetBuffer());

	if(pDataNode == NULL)
	{
		if(strFilename == "")
			return 0;

		Doc.InsertEndChild(*pSaveElement);
		SAFEDEL(pSaveElement)
			return Doc.SaveFile(strFilename.GetBuffer());
	}
	else
	{
		if(strFilename == "")
			return 1;

		Doc.InsertEndChild(*pSaveElement);
		return Doc.SaveFile(strFilename.GetBuffer());
	}

	return 0;
}

void CBitmapFont::Destroy()
{

}

void CBitmapFont::Draw(CString Format, ...)
{
	if(m_pTexture == NULL)
		return;

	if (Format == "")											// If There's No Text
		return;													// Do Nothing

	char Text[256];												// Holds Our CString
	va_list	ap;													// Pointer To List Of Arguments

	va_start(ap, Format);										// Parses The CString For Variables
	vsprintf(Text, Format.GetBuffer(), ap);							// And Converts Symbols To Actual Numbers
	va_end(ap);													// Results Are Stored In Text

	glPushMatrix();
	
	m_pTexture->Bind();
	if(m_pMaterial !=NULL)
	{
		glColor4fv(m_pMaterial->GetAmbient().RGBA);
		// m_pMaterial->Bind();
	}
	else
	{
		glDisable(GL_LIGHTING);
		glColor3d(255, 255, 255);
	}

	BeginBlend();
	BeginOrtho();

	glLoadIdentity();											// Reset The Modelview Matrix

	glTranslated(m_TextPos.x, m_TextPos.y, 0);

	glListBase(m_uiFontBase - 32 + (128*m_uiNI) );				// Choose The Font Set (0 or 1)
	glCallLists(strlen(Text), GL_UNSIGNED_BYTE, Text);			// Write The Text To The Screen
	
	EndOrtho();
	EndBlend();

	glPopMatrix();
}

void CBitmapFont::DrawChar(float x, float y, char c)
{

}

void CBitmapFont::SetTextPos(float X, float Y)
{
	m_TextPos.x = X; m_TextPos.y = Y;
}

tVERTEX2f& CBitmapFont::GetTextPos()
{
	return m_TextPos;
}

UINT CBitmapFont::GetQuadLength()
{
	return m_uiQuadLength;
}

UINT CBitmapFont::GetQuadHeight()
{
	return m_uiQuadHeight;
}

UINT CBitmapFont::GetCharLength()
{
	return m_uiCharLength;
}

UINT CBitmapFont::GetCharHeight()
{
	return m_uiCharHeight;
}

UINT CBitmapFont::GetSpacing()
{
	return m_uiSpacing;
}

bool CBitmapFont::IsOfType(eEntityType eType)
{
	if(eType == Entity_BitmapFont)
		return true;

	return CCustomFont::IsOfType(eType);
}

/*!
	Commences blending mode in OpenGL
*/
void BeginBlend()
{
	glDisable(GL_DEPTH_TEST);							// Disable Depth Testing
	glEnable(GL_BLEND);									// Enable Blending
	glBlendFunc(GL_SRC_ALPHA,GL_ONE);					// Type Of Blending To Perform
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);	// Really Nice Perspective Calculations
	glHint(GL_POINT_SMOOTH_HINT,GL_NICEST);				// Really Nice Point Smoothing
}

/*!
Ends blending mode in OpenGL
*/
void EndBlend()
{
	glBlendFunc(GL_ONE,GL_ZERO);						// Reset to previous settings
	glEnable(GL_DEPTH_TEST);							// (restore)
}

/*!
	Commences orthographic mode.
*/
void BeginOrtho()
{
	int Viewport[4];

	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glPushMatrix();										// Store The Projection Matrix
	glLoadIdentity();									// Reset The Projection Matrix
	glGetIntegerv(GL_VIEWPORT, Viewport);

	glOrtho(0, Viewport[2], 0, Viewport[3], 0, 100);	// Set Up An Ortho Screen
	glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
	glPushMatrix();										// Store The Modelview Matrix
}

/*!
	Finishes orthographic mode.
*/
void EndOrtho()
{
	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glPopMatrix();										// Restore The Old Projection Matrix

	glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
	glPopMatrix();										// Restore The Old Projection Matrix
}