#ifndef BITMAPFONT_H
#define BITMAPFONT_H

#pragma once
#include "CustomFont.h"
#include "TextureManager.h"
#include "MaterialManager.h"

//! Bitmap Font class
/*!
	This class is an implementation of generic font interface class,
	that uses texture to output textured quads with letters.
*/
class CBitmapFont : public CCustomFont
{
	CREATOR(CFontManager);

protected:

	CBitmapFont();
	virtual ~CBitmapFont();

public:
	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that holds data for the animation
		@param strFilename - String containing filename where the data for the animation is stored
	*/
	virtual int LoadXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that will be the parent hold the data for this animation
		@param strFilename - String containing filename where the data for the animation is stored
	*/
	virtual int SaveXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		Clean-up method
	*/
	virtual void Destroy();
	/*!
		Method details:
		@param strFormat - Format of passed parameters 
	*/
	virtual void Draw(CString strFormat, ...);
	/*!
		Method details:
		@param x, y - 2D coordinate to draw character at
		@param c - character (letter or symbol) to draw
	*/
	virtual void DrawChar(float x, float y, char c);
	/*!
		Method details:
		@param uiNormalOrItalic - Is it normal or italic text?
		@param uiCharLength - Character length (in the texture)
		@param uiCharHeight - Character height (in the texture)
		@param uiQuadLength - Quad length for all characters
		@param uiQuadHeight - Quad height for all characters
		@param uiSpacing	- Spacing between quads when drawing them
		\return 1 if created succesfully, 0 otherwise
	*/
	virtual int Create(	UINT uiNormalOrItalic, 
						UINT uiCharLength, 
						UINT uiCharHeight, 
						UINT uiQuadLength, 
						UINT uiQuadHeight, 
						UINT uiSpacing);
	/*!
		Method details:
		@param X, Y - 2D coordinate to start drawing text from
	*/
	void SetTextPos(float X, float Y);
	/*!
		Method details:
		\return Const. Reference to 2D coordinate from where text is being drawn
	*/
	tVERTEX2f& GetTextPos();
	/*!
		Method details:
		\return Quad length (units)
	*/
	UINT GetQuadLength();
	/*!
		Method details:
		\return Quad height (units)
	*/
	UINT GetQuadHeight();
	/*!
		Method details:
		\return Characters length (in pixels)
	*/
	UINT GetCharLength();
	/*!
		Method details:
		\return Characters height (in pixels)
	*/
	UINT GetCharHeight();
	/*!
		Method details:
		\return Spacing between quads (units)
	*/
	UINT GetSpacing();
	/*!
		Method details:
		This is an implementation of Custom RTTI
	*/
	virtual bool IsOfType(eEntityType eType);

protected:
	CTexture *m_pTexture;			/*!< Pointer to the texture */
	CMaterial *m_pMaterial;			/*!< Pointer to the material */
	
	UINT	m_uiNI;					/*!< Normal or italic attribute */
	UINT	m_uiCharLength,			/*!< Character width (in pixels) on the bitmap */
			m_uiCharHeight,			/*!< Character hieght (in pixels) on the bitmap */
			m_uiQuadLength,			/*!< Quad width (in GL units) */
			m_uiQuadHeight,			/*!< Quad height (int GL units) */
			m_uiSpacing;			/*!< Spacing between character quads (GL) */

	UINT	m_uiFontBase;			/*!< GL Font List uiID */
	UINT	m_uiNumLists;			/*!< Number of GL lists */

	tVERTEX2f	m_TextPos;			/*!< 2D coordinate from where to start drawing text */
};

//! Commence blending
void BeginBlend();
//! Finish blending
void EndBlend();

//! Enter orthographic mode
void BeginOrtho();
//! Exit orthographic mode
void EndOrtho();

#endif