#ifndef COMMONSTRUCTS_H
#define COMMONSTRUCTS_H

#pragma once
#include "CommonDefs.h"

struct tRect
{
public:
	union
	{
		struct
		{
			int left;		/*!< Left Attribute */
			int right;		/*!< Right Attribute */
			int top;		/*!< Top Attribute */
			int bottom;		/*!< Bottom Attribute */
		};
		int m_iSide[4];		/*!< An array of above elements that can be accessed by index */
	};

	tRect() {left = right = top = bottom = 0;};
	virtual ~tRect() {};

	tRect(int x1, int x2, int y1, int y2)
	{
		left = x1; right = x2;
		top = y1; bottom = y2;
	};

	/*!
		Assignment operator with constant reference to values parameter
	*/
	const tRect& operator = (const tRect& other)
	{
		left = other.left;
		right = other.right;
		top = other.top;
		bottom = other.bottom;

		return *this;
	};

	/*!
		Index operator
	*/
	const int operator [] (int i)
	{
		if(i == 0)
			return left;
		else if(i == 1)
			return right;
		else if(i == 2)
			return top;
		else if(i == 3)
			return bottom;

		return -1;
	};

	/*!
		Returns width of the rectangle
	*/
	UINT Width()
	{ return right - left; };

	/*!
		Returns height of the rectangle
	*/
	UINT Height()
	{ return top - bottom; };

};

//! Vertex structure (float based)
/*!
	This structure holds 2 float values for 2D vertex.
*/
struct tVERTEX2f
{
	tVERTEX2f(float x = 0, float y = 0) : x(x), y(y){};
	float x;	/*!< X value */
	float y;	/*!< Y value */

	const tVERTEX2f& operator = (const tVERTEX2f& other)
	{
		x = other.x;
		y = other.y;

		return *this;
	};

	const int operator [] (int i)
	{
		if(i == 0)
			return x;
		else if(i == 1)
			return y;

		return -1;
	};
};

//! Vertex structure (float based)
/*!
	This structure holds 2 float values for 2D vertex.
*/
struct tVERTEX2d
{
	tVERTEX2d(int x = 0, int y = 0) : x(x), y(y){};
	int x;	/*!< X value */
	int y;	/*!< Y value */

	const tVERTEX2d& operator = (const tVERTEX2f& other)
	{
		x = other.x;
		y = other.y;

		return *this;
	};

	const int operator [] (int i)
	{
		if(i == 0)
			return x;
		else if(i == 1)
			return y;

		return -1;
	};
};

#endif