#ifndef CUSTOMFONT_H
#define CUSTOMFONT_H

#include "Entity.h"
#include "XMLResource.h"

#pragma once

enum eFontType
{
	Undefined_Font, /*!< Unidentified font */
	Logical_Font,	/*!< Logical font */
	Bitmap_Font		/*!< Bitmap font */
};

enum eTextAlign
{
	Horizontal_Align_Left = 0,		/*!< Align text to the left */
	Horizontal_Align_Center = 1,	/*!< Align text to the middle */
	Horizontal_Align_Right = 2,		/*!< Align text to the right */
	Vertical_Align_Top = 0,			/*!< Align text to the top */
	Vertical_Align_Center = 1,		/*!< Align text to the center */
	Vertical_Align_Bottom = 2		/*!< Align text to the bottom */
};
//! Font class
/*!
	This class provides an interface to higher-abstraction font classes.
*/
class CCustomFont : public CXMLResource
{
	CREATOR(CFontManager);

protected:

	CCustomFont();
	virtual ~CCustomFont();

public:
	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that holds data for the animation
		@param strFilename - String containing filename where the data for the animation is stored
	*/
	virtual int LoadXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that will be the parent hold the data for this animation
		@param strFilename - String containing filename where the data for the animation is stored
	*/
	virtual int SaveXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		Clean-up method
	*/
	virtual void Destroy();
	/*!
		Method details:
		@param strFormat - Format of passed parameters 
	*/
	virtual void Draw(CString strFormat, ...) = 0;
	/*!
		Method details:
		@param x, y - 2D coordinate to draw character at
		@param c - character (letter or symbol) to draw
	*/
	virtual void DrawChar(float x, float y, char c) = 0;

	//! Returns font type
	eFontType GetFontType();
	/*!
		Method details:
		This is an implementation of Custom RTTI
	*/
	virtual bool IsOfType(eEntityType eType);
	
protected:
	DWORD	m_dwFlags;			/*!<  Custom flags */
	eFontType m_eFontType;		/*!<  Font type */
};

#endif