#ifndef ENTITY_H
#define ENTITY_H

#pragma once
#include "Common.h"
#include <atlstr.h>

//! Entity (ROOT) Class
/*!
	This class serves as a base class for all of it's derivatives,
	as well as acting as the RTTI assistant.
*/
class CEntity
{
public:
	CEntity();
	virtual ~CEntity();
	/*!
		Method details:
		@param uiID - The proper Unique Identifier for this entity
	*/
	void SetID(UINT uiID);
	/*!
		Method details:
		\return Unique Identifier for this entity	
	*/
	UINT GetID();
	/*!
		Method details:
		@param strName - Name for this entity
	*/
	void SetName(CString& strName);
	/*!
		Method details:
		\return Name of this entity
	*/
	CString& GetName();
	/*!
		Method details:
		\return RTTI Entity Type of this entity
	*/
	eEntityType GetEntityType();
	
protected:
	/*!
		Method details:
		@param eType - Type of this entity
	*/
	void SetEntityType(eEntityType eType);
	/*!
		Method details:
		This is an implementation of Custom RTTI
	*/
	virtual bool IsOfType(eEntityType eType);

private:
	UINT m_uiID;				/*!< Unique Identifier */
	eEntityType m_EntityType;	/*!< Entity Type (RTTI) */
	CString m_strName;			/*!< Entity' name */
};

#endif