#include "StdAfx.h"
#include "FontManager.h"

CFontManager::CFontManager()
{
	SetEntityType(Entity_FontManager);
	CGlobalLogger::GetSingleton().Write("Font Manager - Instantiated\n");
}

CFontManager::~CFontManager()
{
	Destroy();
	CGlobalLogger::GetSingleton().Write("Font Manager - Destroyed\n");
}

CCustomFont *CFontManager::AddResource(CString strName, UINT uiID, eEntityType ResourceType)
{
	switch(ResourceType)
	{
	case Entity_Default:
	case Entity_CustomFont:
	case Entity_BitmapFont:
		CBitmapFont *pNewBitmapFont = new CBitmapFont;

		pNewBitmapFont->SetID(uiID);
		pNewBitmapFont->SetName(strName);

		push_back(pNewBitmapFont);

		return pNewBitmapFont;
		break;
	}

	return NULL;
}

int CFontManager::LoadXML(TiXmlNode *pDataNode, CString strFilename)
{
	TiXmlElement* pXMLElement = NULL;
	const char *pcValue = NULL;

	int iRetValue = CResourceManager::LoadXML(pDataNode, strFilename);	// LoadXML the data of derivative
	if(iRetValue !=1)
		return iRetValue;

	pXMLElement = m_pXMLElement;

	if(stricmp(pXMLElement->Value(), "Font") == 0)
	{
		CXMLResource *AddedResource = LoadResource(pDataNode, strFilename);
		if(AddedResource !=NULL)
		{
			CGlobalLogger::GetSingleton().Write("Font (Name: %s, ID: %d) has been added successfully.\n", AddedResource->GetName().GetBuffer(), AddedResource->GetID()); 
			return 1;
		}
		else
			return 0;
	}
	// LoadXML Fonts
	UINT FontLoadedCount = 0;
	TiXmlNode *pIterXMLNode = pXMLElement->FirstChild();
	while(pIterXMLNode !=NULL)
	{
		if(stricmp(pIterXMLNode->Value(), "font") == 0)	// We have a Font
		{
			CXMLResource *AddedResource = LoadResource(pIterXMLNode, "");
			if(AddedResource !=NULL)
			{
				CGlobalLogger::GetSingleton().Write("Font (Name: %s, ID: %d) has been added successfully.\n", AddedResource->GetName().GetBuffer(), AddedResource->GetID()); 
				FontLoadedCount++;
			}
		}

		pIterXMLNode = pXMLElement->IterateChildren(pIterXMLNode);
	}

	return FontLoadedCount;
}

int CFontManager::SaveXML(TiXmlNode *pDataNode, CString strFilename)
{
	// Nowhere to save!			Nothing to save!
	if(pDataNode == NULL && strFilename == "")
		return -1;

	char Buf[128] = "";
	TiXmlElement *pSaveXMLElement = NULL;

	if(pDataNode != NULL)
		pSaveXMLElement = pDataNode->ToElement();
	else
		pSaveXMLElement = new TiXmlElement("FontDatabase");

	CCustomFont *pCustomFont = (CCustomFont *)begin();
	set_ptr(pCustomFont);

	while(pCustomFont !=NULL)
	{
		TiXmlElement CustomFontXMLElement("Font");
		if(pCustomFont->SaveXML(&CustomFontXMLElement, ""))
			pSaveXMLElement->InsertEndChild(CustomFontXMLElement);

		pCustomFont = (CCustomFont *)next();
	}

	if(pDataNode == NULL && strFilename !="")
	{
		TiXmlDocument SaveDoc;
		SaveDoc.InsertEndChild(*pSaveXMLElement);
		int iRetValue = SaveDoc.SaveFile(strFilename);
		SAFEDEL(pSaveXMLElement)

			return iRetValue;
	}

	return 1;
}

CCustomFont *CFontManager::LoadResource(TiXmlNode *pDataNode, CString strFilename, UINT uiID, CString strName)
{
	if(pDataNode != NULL)
	{
		const char *pcValue = NULL;
		TiXmlElement *pXMLElement = pDataNode->ToElement();

		pcValue = pXMLElement->Attribute("FontByID");
		if(pcValue !=NULL)
		{
			CCustomFont *ExistingCustomFont = GetResource(-1, atoi(pcValue), "");
			if(ExistingCustomFont !=NULL)
				return ExistingCustomFont;
			else
				return NULL;
		}

		pcValue = pXMLElement->Attribute("FontByName");
		if(pcValue !=NULL)
		{
			CCustomFont *ExistingCustomFont = GetResource(-1, -1, pcValue);
			if(ExistingCustomFont !=NULL)
				return ExistingCustomFont;
			else
				return NULL;
		}
	}

	CCustomFont *NewFont = new CBitmapFont;

	if(NewFont->LoadXML(pDataNode, strFilename))
	{
		if(NewFont->GetID() == 0)
			NewFont->SetID(uiID);

		if(NewFont->GetName() == "")
			NewFont->SetName(strName);

		if(NewFont->GetName() !="")
		{
			if(GetResource(-1, -1, NewFont->GetName()) == NULL)
			{
				push_back(NewFont);
				return NewFont;
			}
			else
			{
				SAFEDEL(NewFont)
					return NULL;
			}
		}
		else
		{
			push_back(NewFont);
			return NewFont;
		}
	}
	else
	{
		SAFEDEL(NewFont)
			return NULL;
	}


	return NULL;
}

CCustomFont *CFontManager::GetResource(int uiIndex, int uiID, CString strName, CString strFilename)
{
	if(uiID !=-1 && uiIndex == -1 && strName == "" && strFilename == "")	// get by uiID
	{
		CXMLResource *ResourcePtr = begin();
		set_ptr(ResourcePtr);

		while(ResourcePtr !=NULL)
		{
			if(ResourcePtr->GetID() == uiID)
				return (CBitmapFont *)ResourcePtr;

			ResourcePtr = next();
		}
	}
	else if(strName !="" && uiIndex == -1 && uiID == -1 && strFilename == "")	// get by name
	{
		CXMLResource *ResourcePtr = begin();
		set_ptr(ResourcePtr);

		while(ResourcePtr !=NULL)
		{
			if(ResourcePtr->GetName() == strName)
				return (CBitmapFont *)ResourcePtr;

			ResourcePtr = next();
		}
	}
	else if(uiIndex !=-1 && uiID == -1 && strName == "" && strFilename == "")	// get by index
	{
		if(uiIndex >=0 && uiIndex < size())
			return (CBitmapFont *)at(uiIndex);
	}
	else if(strFilename !="" && uiIndex ==-1 && uiID == -1 && strName == "")
	{
		CXMLResource *ResourcePtr = begin();
		set_ptr(ResourcePtr);

		while(ResourcePtr !=NULL)
		{
			if(stricmp(ResourcePtr->GetFilename().GetBuffer(), strFilename.GetBuffer()) == 0)
				return (CCustomFont *)ResourcePtr;

			ResourcePtr = next();
		}
	}

	return NULL;
}

int CFontManager::RemoveResource(CCustomFont *Font)
{

	return 1;
}

void CFontManager::Destroy()
{
	if(CGlobalLogger::GetSingleton().GetLogLevel() == 0)
		CGlobalLogger::GetSingleton().Write("Font Manager - Cleaning Up\n");
	else if(CGlobalLogger::GetSingleton().GetLogLevel() == 1)
		CGlobalLogger::GetSingleton().Write("Font Manager - Cleaning Up - Freed %d fonts.\n", size());
	else if(CGlobalLogger::GetSingleton().GetLogLevel() == 2)
	{
		CCustomFont *IterFont = (CCustomFont *)begin();
		set_ptr(IterFont);

		while(IterFont !=NULL)
		{
			if(IterFont->GetFontType() == Bitmap_Font)
				CGlobalLogger::GetSingleton().Write("Bitmap Font released(Name: %s, ID: %d)\n", IterFont->GetName().GetBuffer(), IterFont->GetID());
			else if(IterFont->GetFontType() == Logical_Font)
				CGlobalLogger::GetSingleton().Write("Logical Font released(Name: %s, ID: %d)\n", IterFont->GetName().GetBuffer(), IterFont->GetID());

			IterFont = (CCustomFont *)next();
		}
	}
	CResourceManager::Destroy();
}

bool CFontManager::IsOfType(eEntityType eType)
{
	if(eType == Entity_FontManager)
		return true;

	return CResourceManager::IsOfType(eType);
}
