#ifndef FONTMANAGER_H
#define FONTMANAGER_H

#pragma once
#include "ResourceManager.h"
#include "BitmapFont.h"

//!Animation Manager class
/*!
	This class provides means to managing various fonts (Bitmap or others).
	It manages a collection of fonts (although not pooled yet!) and allow to retrieve
	them by index, name, ID or filename.
*/
class CFontManager : public CResourceManager, public CSingleton<CFontManager>
{
public:
	CFontManager();
	virtual ~CFontManager();

public:
	/*!
		Method details:
		@param pDataNode - Pointer to XML Data node containing data for the font
		@param strFilename - Name of the file containing data for the font
		@param uiID - ID for the font to be loaded
		@param strName - Name for the font to be loaded
		\return  Pointer to the font if it has been loaded succesfully, NULL if not
	*/
	virtual CCustomFont *LoadResource(TiXmlNode *pDataNode, CString strFilename, UINT uiID = 0, CString strName = "");
	/*!
		Method details:
		@param uiIndex - Index by which to retrieve the font
		@param uiID - ID by which to retrieve the font
		@param strName - Name by which to retrieve the font
		@param strFilename - Filename by which to retrieve the font
		\return Pointer to the font if it has been found using given parameters, NULL if not
	*/
	virtual CCustomFont *GetResource(int uiIndex, int uiID = -1, CString strName = "", CString strFilename = "");
	/*!
		Method details:
		@param strName - Name of the font to add
		@param uiID - ID of the font to add
		@param ResourceType - Type of the font to add
		\return Pointer to the font if it has been added succesfully using given parameters, NULL if not
	*/
	virtual CCustomFont *AddResource(CString strName, UINT uiID = 0, eEntityType ResourceType = Entity_Default);
	/*!
		Method details:
		@param pResource - Pointer to the font to remove
		\return 1 if it has been found and removed succesfully, 0 otherwise
	*/
	int RemoveResource(CCustomFont *pFont);

	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that holds data for this texture
		@param strFilename - String containing filename where the data for the font is stored
	*/
	virtual int LoadXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that will be the parent hold the data for this texture
		@param strFilename - String containing filename where the data for the font is stored
	*/
	virtual int SaveXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		This is where clean-up occurs
	*/
	virtual void Destroy();
	/*!
		Method details:
		This is an implementation of Custom RTTI
	*/
	virtual bool IsOfType(eEntityType eType);
};

#endif