#include "StdAfx.h"
#include "GUI.h"

CGUIButton::CGUIButton()
{
	SetEntityType(Entity_GUIButton);
	SetType(GUI_Button);
	Initialize();
}

CGUIButton::~CGUIButton()
{
	SAFEDEL_ARRAY(m_pButtonStates)
	Destroy();
}

void CGUIButton::Initialize()
{
	m_pButtonStates = NULL;
	m_iuStateCount = 0;
	m_iCurrentState = 0;
	m_bLoopState = false;
	m_iCurrentState = 0;
}

int CGUIButton::Create(CGUIElement *pParent, tRect WidgetRect, CCustomFont *pFont, CTexture *pTexture, CMaterial *pMaterial, bool bBorder)
{
	return CGUIStatic::Create(pParent, WidgetRect, pFont, pTexture, pMaterial, bBorder);
}

int CGUIButton::LoadXML(TiXmlNode *pDataNode, CString strFilename)
{
	TiXmlElement* pXMLElement = NULL;
	const char *pcValue = NULL;

	int iRetValue = CGUIStatic::LoadXML(pDataNode, strFilename);	// LoadXML the data of derivative
	if(iRetValue !=1)
		return iRetValue;

	pXMLElement = m_pXMLElement;

	LoadButtonStates(pXMLElement);
	return 1;	// All went OK, return 1
}

int CGUIButton::SaveXML(TiXmlNode *pDataNode, CString strFilename)
{
	// Nowhere to save!			Nothing to save!
	if(pDataNode == NULL && strFilename == "")
		return -1;

	char Buf[128] = "";
	TiXmlElement *pSaveXMLElement = NULL;

	if(pDataNode != NULL)
		pSaveXMLElement = pDataNode->ToElement();
	else
		pSaveXMLElement = new TiXmlElement("Button");

	SaveButtonStates(pSaveXMLElement);

	int iRetValue = CGUIStatic::SaveXML(pSaveXMLElement, strFilename);

	if(pDataNode == NULL && strFilename !="")
	{
		TiXmlDocument SaveDoc;
		SaveDoc.InsertEndChild(*pSaveXMLElement);
		int iRetValue = SaveDoc.SaveFile(strFilename);
		SAFEDEL(pSaveXMLElement)

		return iRetValue;
	}

	return iRetValue;
}

int CGUIButton::SaveButtonStates(TiXmlNode *pDataNode)
{
	if(pDataNode == NULL)
		return -1;

	for (int i=0; i<m_iuStateCount; i++)
	{
		TiXmlElement *pStateXMLElement = new TiXmlElement("State");
		SaveStateEvents(pStateXMLElement, &m_pButtonStates[i]);

		pDataNode->InsertEndChild(*pStateXMLElement);
		SAFEDEL(pStateXMLElement)
	}
	return 1;
}

int CGUIButton::SaveStateEvents(TiXmlNode *pDataNode, tButtonState *uiState)
{
	if(pDataNode == NULL || uiState == NULL)
		return -1;

	for (int i=0; i<uiState->m_iEventCount; i++)
	{
		TiXmlElement *pEventXMLElement = new TiXmlElement("Event");
		if(uiState->m_pEventAnimations[i] !=NULL)
			pEventXMLElement->SetAttribute("AnimByName", uiState->m_pEventAnimations[i]->GetName());

		pDataNode->InsertEndChild(*pEventXMLElement);
		SAFEDEL(pEventXMLElement)
	}
	return 1;
}


void CGUIButton::Draw()
{
	CTextureFrameAnimation *pAnimation = GetCurrentAnimation();
	if(pAnimation !=NULL)
	{
		CTexture *pTexture = pAnimation->GetTexture();
		if(pTexture !=NULL)
		{
			SetTexture(pTexture);

			m_TexCoord[0].x = (float)pAnimation->GetCurrentFrame()->left / (float)pTexture->GetWidth();
			m_TexCoord[1].x = (float)pAnimation->GetCurrentFrame()->right / (float)pTexture->GetWidth();
			m_TexCoord[2].x = (float)pAnimation->GetCurrentFrame()->right / (float)pTexture->GetWidth();
			m_TexCoord[3].x = (float)pAnimation->GetCurrentFrame()->left / (float)pTexture->GetWidth();

			//if(pAnimation->GetCurrentFrame()->m_iBottom != pTexture->GetHeight())
			//{
			m_TexCoord[0].y = 1.0f - ((float)pAnimation->GetCurrentFrame()->bottom / (float)pTexture->GetHeight());
			m_TexCoord[1].y = 1.0f - ((float)pAnimation->GetCurrentFrame()->bottom / (float)pTexture->GetHeight());
			m_TexCoord[2].y = 1.0f - ((float)pAnimation->GetCurrentFrame()->top / (float)pTexture->GetHeight());
			m_TexCoord[3].y = 1.0f - ((float)pAnimation->GetCurrentFrame()->top / (float)pTexture->GetHeight());
		}

		pAnimation->Animate(100);
	}

	CGUIStatic::Draw();
}

CTextureFrameAnimation *CGUIButton::GetEventAnimation(UINT uiEventIndex)
{
	// Basically, goes through all events and states and finds one appropriate
	// for current config of 'm_iButtonState' and 'event' variables
	// and if finds one, returns its animation
	if(m_pButtonStates !=NULL)
	{
		if(uiEventIndex >= 0 && uiEventIndex < m_pButtonStates[GetState()].m_iEventCount)
			return m_pButtonStates[GetState()].m_pEventAnimations[uiEventIndex];
	}

	return NULL;	
}


UINT CGUIButton::GetState()
{
	return m_iCurrentState;
}

void CGUIButton::SetState(bool bLoopAnimation, UINT uiState)
{
	if(uiState >=0 && uiState < m_iuStateCount)
		m_iCurrentState = uiState;

	m_bLoopState = bLoopAnimation;
}

CTextureFrameAnimation *CGUIButton::GetCurrentAnimation()
{
	UINT EventType = 0; 
	// If no mouse over this button and its not pressed

	// uiState 0 - NORMAL state
	// A button MUST have a normal state!
	// There must be no change of flags when a button is disabled!
	// Normal (not mouseover)
	if(PointInRect(
		GetRect(),
		CInputEngine::GetSingleton().GetMouse()->GetCursorPos().x, 
		CInputEngine::GetSingleton().GetMouse()->GetCursorPos().y) && 
		CInputEngine::GetSingleton().GetMouse()->IsButtonDown(VK_LBUTTON) == false)		// Only mouseover
	{
		if(GetEventAnimation(Button_MouseOver) !=NULL)
			return GetEventAnimation(Button_MouseOver);
	}
	else if(PointInRect(
		GetRect(),
		CInputEngine::GetSingleton().GetMouse()->GetCursorPos().x,
		CInputEngine::GetSingleton().GetMouse()->GetCursorPos().y) && 
		CInputEngine::GetSingleton().GetMouse()->IsButtonDown(VK_LBUTTON))		// LMouseDown
	{
		if(GetEventAnimation(Button_MouseDown) !=NULL)
			return GetEventAnimation(Button_MouseDown);
	}

	return GetEventAnimation(Button_Default);
}

bool CGUIButton::LoadButtonStates(TiXmlNode *pButtonXMLNode)
{
	UINT StateCount = 0;

	if(pButtonXMLNode == NULL)
		return false;

	TiXmlNode *pIterXMLNode = pButtonXMLNode->FirstChild();
	while(pIterXMLNode !=NULL)
	{
		if(stricmp(pIterXMLNode->Value(), "state") == 0)
			StateCount++;

		pIterXMLNode = pButtonXMLNode->IterateChildren(pIterXMLNode);
	}

	if(StateCount !=0)
	{
		SAFEDEL_ARRAY(m_pButtonStates)
		m_iuStateCount = StateCount;
	}
	else
	{
		CGlobalLogger::GetSingleton().Write("\t**********General Warning\t**********\n");
		CGlobalLogger::GetSingleton().Write("Button (Name: %s, ID: %d) has 0 States (uiState XML Node(s)).", GetName(), GetID());

		return true;
	}

	if(m_iuStateCount > 0)
	{
		UINT Count = 0;
		m_pButtonStates = new tButtonState[m_iuStateCount];

		pIterXMLNode = pButtonXMLNode->FirstChild();
		while(pIterXMLNode !=NULL)
		{
			if(stricmp(pIterXMLNode->Value(), "state") == 0)
			{
				if(!LoadStateEvents(&m_pButtonStates[Count], pIterXMLNode))
				{
					CGlobalLogger::GetSingleton().Write("\n\t\t!!!!!!!!!!General Error!!!!!!!!!!!!\n");
					CGlobalLogger::GetSingleton().Write("Button (Name: %s, ID: %d) cannot parse uiState(%d) events. Check syntax.", GetName(), GetID(), Count);

					return false;
				}
			}

			pIterXMLNode = pButtonXMLNode->IterateChildren(pIterXMLNode);
			Count++;
		}
	}

	return true;
}

bool CGUIButton::LoadStateEvents(tButtonState *uiState, TiXmlNode *pStateXMLNode)
{
	if(uiState == NULL || pStateXMLNode == NULL)
		return false;

	UINT EventCount = 0;

	TiXmlNode *pIterXMLNode = pStateXMLNode->FirstChild();
	while(pIterXMLNode !=NULL)
	{
		if(stricmp(pIterXMLNode->Value(), "event") == 0)
			EventCount++;

		pIterXMLNode = pStateXMLNode->IterateChildren(pIterXMLNode);
	}

	UINT uiIndex = 0;
	uiState->m_iEventCount = EventCount;
	if(EventCount > 0)
	{
		uiState->m_pEventAnimations = new CTextureFrameAnimation*[EventCount];
		for (int i=0; i<EventCount; i++)
			uiState->m_pEventAnimations[i] = NULL;

		pIterXMLNode = pStateXMLNode->FirstChild();
		while(pIterXMLNode !=NULL)
		{
			CAnimation *pAnimation = CAnimationManager::GetSingleton().LoadResource(pIterXMLNode, "");
			if(pAnimation !=NULL)
			{
				if(pAnimation->GetEntityType() == Entity_TextureFrameAnimation)
				{
					uiState->m_pEventAnimations[uiIndex] = (CTextureFrameAnimation *)pAnimation;
					uiIndex++;
				}
			}
			else
				uiState->m_pEventAnimations[uiIndex] = NULL;

			pIterXMLNode = pStateXMLNode->IterateChildren(pIterXMLNode);
		}
	}

	return true;
}

int CGUIButton::OnLMouseUp(UINT x, UINT y)
{
	CGUIElement *IterElement = end();
	set_ptr(IterElement);

	while(IterElement !=NULL)
	{
		if(IterElement->OnLMouseUp(x, y))	// If one element finds use to the mouse_down event, do not let others to use it
			return 1;
		IterElement = prev();
	}

	if(GetType() !=GUI)
	{
		if(Visible())
		{
			if(PointInRect(GetRect(), x, y))
			{
				if(GetGUI()->GetActiveElement() == this) // if mouse was down initially on this element
				{
					if(m_iuStateCount > 0)
					{
						// Progress in button states
						if(m_iCurrentState == (m_iuStateCount - 1))
						{
							if(m_bLoopState)
								m_iCurrentState = 0;
						}
						else
							m_iCurrentState++;
					}

					// Broadcast the 'pressed' message
					GetGUI()->PostMessage(this, NULL, GUI_Message_ButtonPressed);
				}
				return CGUIStatic::OnLMouseUp(x, y);
			}
		}
	}

	return CGUIStatic::OnLMouseUp(x, y);
}

void CGUIButton::Destroy()
{
	CGUIStatic::Destroy();
}

bool CGUIButton::IsOfType(eEntityType eType)
{
	if(eType == Entity_GUIButton)
		return true;

	return CGUIStatic::IsOfType(eType);
}

