// GUIMenu.cpp: implementation of the CGUIDropList class.
//
//////////////////////////////////////////////////////////////////////
#include "StdAfx.h"
#include "GUI.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CGUIDropList::CGUIDropList()
{
	Initialize();
}

CGUIDropList::~CGUIDropList()
{
	
}

void CGUIDropList::Initialize()
{
	SetType(GUI_DropList);
	m_iListType = 2;
	m_bIdentSize = true;
	m_iCurSel = -1;
	m_bExpanded = false;
	m_iSpacing = 1;
}

int CGUIDropList::Create(CGUIElement *pParent, tRect WidgetRect, CCustomFont *pFont, CTexture *pTexture, CMaterial *pMaterial, bool bBorder, CGUIButton *pDefaultEntryBtn)
{
	if(!CGUIButton::Create(pParent, WidgetRect, pFont, pTexture, pMaterial, bBorder))
		return 0;

	if(pDefaultEntryBtn !=NULL)
	{
		if(!IsChild(pDefaultEntryBtn))
		{
			pDefaultEntryBtn->SetParent(this);
			pDefaultEntryBtn->RemoveChild(pDefaultEntryBtn->GetChild(GUI_Border));

			push_back(pDefaultEntryBtn);
		}
	}

	return Resize(WidgetRect);
}

void CGUIDropList::Expand()
{
	// Shows all buttons (e.g expand)
	for (UINT i=0; i<GetChildCount(); i++)
	{
		CGUIElement *pElement = GetChild(i);
		if(pElement->GetType() == GUI_Button)
		{
			((CGUIButton *)pElement)->Show();
			((CGUIButton *)pElement)->ShowText();
		}
	}

	m_bExpanded = true;
}

void CGUIDropList::Collapse()
{
	// Hides all buttons
	for (UINT i=0; i<GetChildCount(); i++)
	{
		CGUIElement *pElement = GetChild(i);
		if(pElement->GetType() == GUI_Button)
		{
			((CGUIButton *)pElement)->Hide();
			((CGUIButton *)pElement)->HideText();
		}
	}

	// Sets flag
	m_bExpanded = false;
}

void CGUIDropList::Draw()
{
	CGUIElement *pIter = begin();
	set_ptr(pIter);

	while(pIter !=NULL)
	{
		if(pIter->GetChild(GUI_Border) !=NULL)
			pIter->RemoveChild(pIter->GetChild(GUI_Border));

		pIter = next();
	}

	CGUIButton::Draw();
}

int CGUIDropList::LoadXML(TiXmlNode *pDataNode, CString strFilename)
{
	TiXmlElement* pXMLElement = NULL;
	const char *pcValue = NULL;

	int iRetValue = CGUIButton::LoadXML(pDataNode, strFilename);	// LoadXML the data of derivative
	if(iRetValue !=1)
		return iRetValue;

	pXMLElement = m_pXMLElement;

	pcValue = pXMLElement->Attribute("Type");
	if(pcValue !=NULL)
	{
		m_iListType = atoi(pcValue);
		pcValue = NULL;
	}
	else
	{
		CGlobalLogger::GetSingleton().Write("\t**********General Warning\t**********\n");
		CGlobalLogger::GetSingleton().Write("DropList (Name: %s, ID: %d) doesn't have Type (Type) Attribute specified. Assuming default...", GetName(), GetID());
	}

	pcValue = pXMLElement->Attribute("CurSel");
	if(pcValue !=NULL)
	{
		SetCurSel(atoi(pcValue));
		pcValue = NULL;
	}
	else
	{
		CGlobalLogger::GetSingleton().Write("\t**********General Warning\t**********\n");
		CGlobalLogger::GetSingleton().Write("DropList (Name: %s, ID: %d) doesn't have Current iSelection (CurSel) Attribute specified. Assuming default...", GetName(), GetID());
	}

	pcValue = pXMLElement->Attribute("Expand");
	if(pcValue !=NULL)
	{
		m_bExpanded = (bool)atoi(pcValue);
		pcValue = NULL;
	}

	CGUIElement *pIterElement = begin();
	set_ptr(pIterElement);

	while(pIterElement !=NULL)
	{
		if(pIterElement->GetType() == GUI_Button)
		{
			CGUIButton *pButton = (CGUIButton *)pIterElement;
			pButton->RemoveChild(pButton->GetChild(GUI_Border));
		}

		pIterElement = next();
	}

	if(m_bExpanded)
		Expand();
	else
		Collapse();

	return iRetValue;
}

int CGUIDropList::SaveXML(TiXmlNode *pDataNode, CString strFilename)
{		
	// Nowhere to save!			Nothing to save!
	if(pDataNode == NULL && strFilename == "")
		return -1;

	char Buf[128] = "";
	TiXmlElement *pSaveXMLElement = NULL;

	if(pDataNode != NULL)
		pSaveXMLElement = pDataNode->ToElement();
	else
		pSaveXMLElement = new TiXmlElement("DropList");

	pSaveXMLElement->SetAttribute("Type", m_iListType);
	pSaveXMLElement->SetAttribute("CurSel", m_iCurSel);
	pSaveXMLElement->SetAttribute("Expand", (int)m_bExpanded);

	int iRetValue = CGUIButton::SaveXML(pSaveXMLElement, strFilename);

	if(pDataNode == NULL && strFilename !="")
	{
		TiXmlDocument SaveDoc;
		SaveDoc.InsertEndChild(*pSaveXMLElement);
		int iRetValue = SaveDoc.SaveFile(strFilename);
		SAFEDEL(pSaveXMLElement)

		return iRetValue;
	}

	return iRetValue;
}

int CGUIDropList::Resize(tRect& NewRect)
{
	UINT MaxWidth = 0;

	// Find longest text entry size
	for (UINT i=0; i<GetChildCount(); i++)
	{
		CGUIElement *pElement = GetChild(i);

		if(pElement->GetType() == GUI_Button)
		{
			if(((CGUIButton *)pElement)->GetText() !="")
			{
				UINT TotalWidth = 0;
				for (UINT j=0; j<((CGUIButton *)pElement)->GetLineCount(); j++)
				{
					CBitmapFont *pBitmapFont = (CBitmapFont *)((CGUIButton *)pElement)->GetFont();
					if(pBitmapFont !=NULL)
					{
						UINT FontLetterWidth = pBitmapFont->GetQuadLength() + pBitmapFont->GetSpacing();
						TotalWidth = ((CGUIButton *)pElement)->GetLineLength(j) * FontLetterWidth;

						if(MaxWidth <= TotalWidth)
							MaxWidth = TotalWidth;
					}
				}
			}
		}

		if(m_iListType !=1)
		{
			// Get absolute maximum width
			if(MaxWidth < GetWidth() || MaxWidth > GetWidth())
				MaxWidth = GetWidth();
		}

		// Figure out sub-parameters
		if(m_iListType == 0) // All entries are of the same width!
		{
			//LimitWidth(true);
			m_bIdentSize = true; 
		}
		else if(m_iListType == 1)	// A droplist width becomes the width of the 'lengthiest' button
			// and all entries are of the same width!
		{
			tRect CurrentRect = GetRect();
			CurrentRect.right = CurrentRect.left + MaxWidth;
			SetRect(CurrentRect);
			m_bIdentSize = true; 
		}
		else if(m_iListType == 2)	// A droplist width becomes the width of the 'widthest' button 
			// but all entries stay with their own width!
		{
			tRect CurrentRect = GetRect();
			CurrentRect.right = CurrentRect.left + MaxWidth;
			SetRect(CurrentRect);
			m_bIdentSize = false;
		}
	}
	// OK, we got max length, assign each menu its rectangle
	tRect PreviousRect = GetRect();
	for (UINT j=0; j<GetChildCount(); j++)
	{
		CGUIElement *pElement = GetChild(j);

		if(pElement->GetType() == GUI_Button)
		{
			// We store previous rect so that next button will know its rectangle coords
			UINT YCount = 0;
			
			UINT CurrentWidth = MaxWidth;
			if(!m_bIdentSize)
			{
				UINT Width = 0;
				for (UINT i=0; i<((CGUIButton *)pElement)->GetLineCount(); i++)
				{
					CBitmapFont *pBitmapFont = (CBitmapFont *)((CGUIButton *)pElement)->GetFont();
					if(pBitmapFont !=NULL)
					{
						UINT FontLetterWidth = pBitmapFont->GetQuadLength() + pBitmapFont->GetSpacing();

						Width = ((CGUIButton *)pElement)->GetLineLength(i) * FontLetterWidth;
						if(CurrentWidth < Width)
							CurrentWidth = Width;
					}
				}
				CurrentWidth = Width;


			}
			CBitmapFont *pBitmapFont = (CBitmapFont *)((CGUIButton *)pElement)->GetFont();

			tRect NewElementRect;
			NewElementRect.left = PreviousRect.left;
			NewElementRect.right = PreviousRect.left + CurrentWidth;
			NewElementRect.top = PreviousRect.bottom - m_iSpacing;
			if(pBitmapFont == NULL)
				NewElementRect.bottom = NewElementRect.top - 20 - 1;
			else
				NewElementRect.bottom = NewElementRect.top - pBitmapFont->GetQuadHeight() - 1;

			pElement->SetRect(NewElementRect);
			PreviousRect = NewElementRect;

			YCount++;
		}
	}

	return CGUIElement::Resize(NewRect);
}

void CGUIDropList::ProcessMessage(tGUIMessage& Message)
{
	switch(Message.uiMessage)
	{
		// On Movement/resize we re-calculate our buttons rectangles!
	case GUI_Message_MoveX:
	case GUI_Message_MoveY:
	case GUI_Message_MoveXY:

		if(Message.pSender == this)
			Resize(GetRect());
		break;

	case GUI_Message_ButtonPressed:
		{
			switch(Message.pSender->GetType())
			{
			case GUI_DropList:
				{
					if(Message.pSender == this) // If the button was this droplist
					{
						if(m_bExpanded)	// Do its stuff
							Collapse();
						else
							Expand();
					}
					break;
				}

			case GUI_Button:
				{
					// If the button was the child of this droplist
					if(!IsChild(Message.pSender))
						break;

					UINT Count = 0;
					for (UINT i=0; i<GetChildCount(); i++)
					{
						CGUIElement *pElement = GetChild(i);

						if(pElement->GetType() == GUI_Button)	// Find out which button has sent it (by count)
						{
							if(Message.pSender == pElement)
							{
								SetCurSel(Count);		// And set the selection count index
								Collapse();
							}
							Count++;
						}
					}
					break;
				}
			}
		}
	}
}

void CGUIDropList::SetCurSel(int iSelection)
{
	if(iSelection >=0)
	{
		UINT Count = 0;
		CGUIElement *pIterElement = begin();
		set_ptr(pIterElement);

		while(pIterElement !=NULL)
		{
			if(pIterElement->GetType() == GUI_Button)
			{
				if(Count == iSelection)
				{
					m_iCurSel = iSelection;
					SetText(((CGUIButton *)pIterElement)->GetText());
				}
				Count++;
			}

			pIterElement = next();
		}
	}
}

int CGUIDropList::GetCurSel()
{
	return m_iCurSel;
}

bool CGUIDropList::IsOfType(eEntityType eType)
{
	if(eType == Entity_GUIDropList)
		return true;

	return CGUIButton::IsOfType(eType);
}