#include "StdAfx.h"
#include "GUI.h"

CGUIElement::CGUIElement()
{
	SetEntityType(Entity_GUIElement);
	Initialize();
	
	m_pParent = NULL;
	SetType(GUI_Element);
	m_pTexture = NULL;
	m_pMaterial = NULL;
}

CGUIElement::~CGUIElement()
{
	Destroy();
}

void CGUIElement::Initialize()
{
	m_pGUI = NULL;
	m_dwFlags = 0;
	m_pMaterial = NULL;
	m_pTexture = NULL;

	m_TexCoord[0].x = 0;
	m_TexCoord[0].y = 1;

	m_TexCoord[1].x = 1;
	m_TexCoord[1].y = 1;

	m_TexCoord[2].x = 1;
	m_TexCoord[2].y = 0;

	m_TexCoord[3].x = 0;
	m_TexCoord[3].y = 0;

	m_bAutoCalc = true;
	m_bVisible = true;
}

int CGUIElement::Create(CGUIElement *pParent, tRect WidgetRect, CTexture *pTexture, CMaterial *pMaterial, bool bBorder)
{
	if(pParent == NULL)
		return 0;

	SetParent(pParent);
	SetRect(WidgetRect);

	if(m_pTexture == NULL && pTexture !=NULL)
		m_pTexture = pTexture;

	if(m_pMaterial == NULL && pMaterial !=NULL)
		m_pMaterial = pMaterial;

	if(GetGUI() !=NULL)
	{
		//if(bBorder && GetChild(GUI_Border) == NULL)
		//	push_back(GetGUI()->AddBorder());
	}

	return 1;
}

int CGUIElement::LoadXML(TiXmlNode *pDataNode, CString strFilename)
{
	TiXmlElement* pXMLElement = NULL;
	const char *pcValue = NULL;

	int iRetValue = CXMLResource::LoadXML(pDataNode, strFilename);	// LoadXML the data of derivative
	if(iRetValue !=1)
		return iRetValue;

	pXMLElement = m_pXMLElement;

	pcValue = pXMLElement->Attribute("Rect");
	if(pcValue !=NULL)
	{
		sscanf(pcValue, "%d,%d,%d,%d", &m_Rect.left, &m_Rect.right, &m_Rect.top, &m_Rect.bottom);
		pcValue = NULL;
	}
	else
	{
		pcValue = pXMLElement->Attribute("XPos");
		if(pcValue !=NULL)
		{
			m_Rect.left = atoi(pcValue);
			pcValue = NULL;
		}
		else
		{
			CGlobalLogger::GetSingleton().Write("\n\t\t!!!!!!!!!!General Error!!!!!!!!!!!!\n");
			CGlobalLogger::GetSingleton().Write("Element (Type: %s, Name: %s, ID: %d) doesn't have Rectangle X Position in absence of \"Rect\" Attribute.", GetElementType(), GetName(), GetID());
		}

		pcValue = pXMLElement->Attribute("YPos");
		if(pcValue !=NULL)
		{
			m_Rect.top = atoi(pcValue);
			pcValue = NULL;
		}
		else
		{
			CGlobalLogger::GetSingleton().Write("\n\t\t!!!!!!!!!!General Error!!!!!!!!!!!!\n");
			CGlobalLogger::GetSingleton().Write("Element (Type: %s, Name: %s, ID: %d) doesn't have Rectangle Y Position in absence of \"Rect\" Attribute.", GetElementType(), GetName(), GetID());
		}

		pcValue = pXMLElement->Attribute("Width");
		if(pcValue !=NULL)
		{
			m_Rect.right = m_Rect.left + atoi(pcValue);
			pcValue = NULL;
		}
		else
		{
			CGlobalLogger::GetSingleton().Write("\n\t\t!!!!!!!!!!General Error!!!!!!!!!!!!\n");
			CGlobalLogger::GetSingleton().Write("Element (Type: %s, Name: %s, ID: %d) doesn't have Width Attribute in absence of \"Rect\" Attribute.", GetElementType(), GetName(), GetID());
		}

		pcValue = pXMLElement->Attribute("Height");
		if(pcValue !=NULL)
		{
			m_Rect.bottom = m_Rect.top - atoi(pcValue);
			pcValue = NULL;
		}
		else
		{
			CGlobalLogger::GetSingleton().Write("\n\t\t!!!!!!!!!!General Error!!!!!!!!!!!!\n");
			CGlobalLogger::GetSingleton().Write("Element (Type: %s, Name: %s, ID: %d) doesn't have Height Attribute in absence of \"Rect\" Attribute.", GetElementType(), GetName(), GetID());
		}
	}

	pcValue = pXMLElement->Attribute("MovementFlags");
	if(pcValue !=NULL)
	{
		int RestrictX = 0, RestrictY = 0;
		sscanf(pcValue, "%d,%d", &RestrictX, &RestrictY);

		if(RestrictX)
			SetFlag(GUI_Flag_RestrictMovementX, true);

		if(RestrictY)
			SetFlag(GUI_Flag_RestrictMovementX, true);

		pcValue = NULL;
	}

	pcValue = pXMLElement->Attribute("ResizeFlags");
	if(pcValue !=NULL)
	{
		int ResizeX = 0, ResizeY = 0;
		sscanf(pcValue, "%d,%d", &ResizeX, &ResizeY);

		if(ResizeX)
			SetFlag(GUI_Flag_RestrictSizeX, true);

		if(ResizeY)
			SetFlag(GUI_Flag_RestrictSizeY, true);

		pcValue = NULL;
	}

	pcValue = pXMLElement->Attribute("UV");
	if(pcValue !=NULL)
	{
		sscanf(pcValue, "(%f,%f),(%f,%f),(%f,%f),(%f,%f)", 
			&m_TexCoord[0].x, &m_TexCoord[0].y,
			&m_TexCoord[1].x, &m_TexCoord[1].y, 
			&m_TexCoord[2].x, &m_TexCoord[2].y, 
			&m_TexCoord[3].x, &m_TexCoord[3].y);

		pcValue = NULL;
	}

	CTexture *pNewTexture = CTextureManager::GetSingleton().LoadResource(m_pXMLElement, "");
	if(pNewTexture !=NULL)
		m_pTexture = pNewTexture;

	CMaterial *pNewMaterial = CMaterialManager::GetSingleton().LoadResource(m_pXMLElement, "");
	if(pNewMaterial !=NULL)
		m_pMaterial = pNewMaterial;

	// LoadXML widgets
	UINT WidgetCount = 0;
	TiXmlNode *pIterXMLNode = pXMLElement->FirstChild();
	while(pIterXMLNode !=NULL)
	{
		CGUIElement *NewWidget = NULL;
		if(stricmp(pIterXMLNode->Value(), "Element") == 0)
			NewWidget = new CGUIElement;
		else if(stricmp(pIterXMLNode->Value(), "Border") == 0)
			NewWidget = new CGUIBorder;
		else if(stricmp(pIterXMLNode->Value(), "Static") == 0)
			NewWidget = new CGUIStatic;
		else if(stricmp(pIterXMLNode->Value(), "Button") == 0)
			NewWidget = new CGUIButton;
		else if(stricmp(pIterXMLNode->Value(), "RadioGroup") == 0)
			NewWidget = new CGUIRadioGroup;
		else if(stricmp(pIterXMLNode->Value(), "DropList") == 0)
			NewWidget = new CGUIDropList;
		
		if(NewWidget !=NULL)
		{
			NewWidget->SetParent(this);

			if(NewWidget->LoadXML(pIterXMLNode, ""))
			{
				push_back(NewWidget);
				WidgetCount++;
			}
		}

		pIterXMLNode = pXMLElement->IterateChildren(pIterXMLNode);
	}

	Resize(m_Rect);
	return 1;	// All went OK, return 1
}

int CGUIElement::SaveXML(TiXmlNode *pDataNode, CString strFilename)
{
	// Nowhere to save!			Nothing to save!
	if(pDataNode == NULL && strFilename == "")
		return -1;

	char Buf[128] = "";
	TiXmlElement *pSaveXMLElement = NULL;

	if(pDataNode != NULL)
		pSaveXMLElement = pDataNode->ToElement();
	else
		pSaveXMLElement = new TiXmlElement("Element");

	// Parse our element' rectangle data
	if(GetParent() !=NULL)
	{
		if(GetParent()->GetType() !=GUI_Border)
		{
			if(!(m_Rect.left == 0 && m_Rect.right == 0 && m_Rect.top == 0 && m_Rect.bottom == 0))
			{
				sprintf(Buf, "%i,%i,%i,%i", m_Rect.left, m_Rect.right, m_Rect.top, m_Rect.bottom);
				pSaveXMLElement->SetAttribute("Rect", Buf);

				memset(Buf, 0, sizeof(char) * 128);
			}
		}
	}

	if(GetName() !="")
		pSaveXMLElement->SetAttribute("Name", GetName());

	pSaveXMLElement->SetAttribute("Visible", (int)Visible());

	if(GetID() !=0)
	{
		sprintf((char *)Buf, "%d", GetID());
		pSaveXMLElement->SetAttribute("uiID", Buf);

		memset(Buf, 0, sizeof(char) * 128);
	}

	memset(Buf, 0, sizeof(char) * 128);

	bool bAllowMoveX = IsFlagSet(GUI_Flag_RestrictMovementX);
	bool bAllowMoveY = IsFlagSet(GUI_Flag_RestrictMovementY);

	sprintf(Buf, "%d,%d", (int)!bAllowMoveX, (int)!bAllowMoveY);
	pSaveXMLElement->SetAttribute("MovementFlags", (char *)Buf);

	bool bAllowSizeX = IsFlagSet(GUI_Flag_RestrictSizeX);
	bool bAllowSizeY = IsFlagSet(GUI_Flag_RestrictSizeY);

	sprintf(Buf, "%d,%d", (int)!bAllowSizeX, (int)!bAllowSizeY);
	pSaveXMLElement->SetAttribute("ResizeFlags", (char *)Buf);

	memset(Buf, 0, sizeof(char) * 128);

#ifdef GUI_USE_ACTIVE_INACTIVE

	if(m_ActiveColor != GUI_DEFAULT_ACTIVE_COLOR)
	{
		sprintf(Buf, "%d,%d,%d", m_ActiveColor.r, m_ActiveColor.g, m_ActiveColor.b);
		pSaveXMLElement->SetAttribute("ActiveColor", Buf);
	}

	if(m_InactiveColor !=GUI_DEFAULT_INACTIVE_COLOR)
	{
		memset(Buf, 0, sizeof(char) * 128);
		sprintf(Buf, "%d,%d,%d", m_InactiveColor.r, m_InactiveColor.g, m_InactiveColor.b);
		pSaveXMLElement->SetAttribute("InactiveColor", Buf);
	}
#else
	if(m_pMaterial !=NULL)
		pSaveXMLElement->SetAttribute("MatByName", m_pMaterial->GetName());
#endif

	if(m_pTexture !=NULL)
		pSaveXMLElement->SetAttribute("TexByName", m_pTexture->GetName());

	if((m_TexCoord[0].x !=0 && m_TexCoord[0].y !=1) &&
		(m_TexCoord[1].x !=1 && m_TexCoord[1].y !=1) &&
		(m_TexCoord[2].x !=1 && m_TexCoord[2].y !=0) &&
		(m_TexCoord[3].x !=0 && m_TexCoord[3].y !=0))
	{
		sprintf(Buf, "(%f,%f),(%f,%f),(%f,%f),(%f,%f)", 
			m_TexCoord[0].x, m_TexCoord[0].y,
			m_TexCoord[1].x, m_TexCoord[1].y, 
			m_TexCoord[2].x, m_TexCoord[2].y, 
			m_TexCoord[3].x, m_TexCoord[3].y);

		pSaveXMLElement->SetAttribute("UV", Buf);

		memset(Buf, 0, sizeof(char) * 128);
	}

	CGUIElement *pIterElement = begin();
	set_ptr(pIterElement);

	while(pIterElement !=NULL)
	{
		TiXmlElement SaveChildXMLElement(pIterElement->GetElementType());
		pIterElement->SaveXML(&SaveChildXMLElement, "");

		pSaveXMLElement->InsertEndChild(SaveChildXMLElement);
		pIterElement = next();
	}

	if(pDataNode == NULL && strFilename !="")
	{
		TiXmlDocument SaveDoc;
		SaveDoc.InsertEndChild(*pSaveXMLElement);
		int iRetValue = SaveDoc.SaveFile(strFilename);
		SAFEDEL(pSaveXMLElement)

		return iRetValue;
	}

	return 1;
}

int CGUIElement::OnLMouseDown(UINT x, UINT y)
{
	CGUIElement *IterElement = end();
	set_ptr(IterElement);

	while(IterElement !=NULL)
	{
		if(IterElement->OnLMouseDown(x, y))	// If one element finds use to the mouse_down event, do not let others to use it
			return 1;
		IterElement = prev();
	}

	if(GetType() !=GUI)	// process it if its not a GUI
	{
		if(PointInRect(GetRect(), x, y))	// If the click was inside the rectangle
		{
			GetGUI()->SetActiveElement(this);	// Set this as an active element. This also calls PostMessage in AquireFocus
			GetGUI()->Lock();
			return 1;
		}
	}

	return 0;
}

int CGUIElement::OnLMouseUp(UINT x, UINT y)
{
	CGUIElement *IterElement = end();
	set_ptr(IterElement);

	while(IterElement !=NULL)
	{
		if(IterElement->OnLMouseUp(x, y))	// If one element finds use to the mouse_down event, do not let others to use it
			return 1;
		IterElement = prev();
	}

	if(GetType() !=GUI)
	{
		if(PointInRect(GetRect(), x, y))
		{
			GetGUI()->PostMessage(this, NULL, GUI_Message_AquireFocus);
			return 1;
		}
	}

	return 0;
}

int CGUIElement::OnMouseMove(UINT x, UINT y)
{
	if(x == 0 && y == 0)
		return 0;

	int iRetValue = 0;
	CGUIElement *IterElement = end();
	set_ptr(IterElement);

	while(IterElement !=NULL)
	{
		iRetValue|=IterElement->OnMouseMove(x, y);
		IterElement = prev();
	}

	if(GetType() !=GUI)
	{
		if(Visible())
		{
			// Move this element if it has a lmousedown flag
			if(GetGUI()->IsLocked())
			{
				tVERTEX2d CursorPos = CInputEngine::GetSingleton().GetMouse()->GetCursorPos();
				tVERTEX2d PrevCursorPos = CInputEngine::GetSingleton().GetMouse()->GetPrevCursorPos();

				if(GetGUI()->GetActiveElement() == this)
				{
					OnMove(CursorPos.x - PrevCursorPos.x, CursorPos.y - PrevCursorPos.y);
					return 1;
				}
			}

			if(PointInRect(m_Rect, x, y))
				iRetValue = 1;
		}
	}

	return iRetValue;
}


int CGUIElement::OnRMouseDown(UINT x, UINT y)
{

	return 0;
}

int CGUIElement::OnRMouseUp(UINT x, UINT y)
{

	return 0;
}

int CGUIElement::OnMove(UINT x, UINT y)
{
	if(GetType() == GUI)
		return 0;

	int mx = GUI_Message_Default, my = GUI_Message_Default;
	if(Visible())
	{	
		// Resize this element according to its restriction flags
		if(!IsFlagSet(GUI_Flag_RestrictMovementX) && (x !=0))
		{
			m_Rect.left+= x;
			m_Rect.right+= x;
			mx = GUI_Message_MoveX;
		}

		if(!IsFlagSet(GUI_Flag_RestrictMovementY) && (y !=0))
		{
			m_Rect.top+= y;
			m_Rect.bottom+= y;
			my = GUI_Message_MoveY;
		}
	}

	if(mx != GUI_Message_Default && my != GUI_Message_Default)
	{
		GetGUI()->PostMessage(this, NULL, GUI_Message_MoveXY, x, y);
		return GUI_Message_MoveXY;
	}
	else if(mx == GUI_Message_MoveX && my == GUI_Message_Default)
	{
		GetGUI()->PostMessage(this, NULL, GUI_Message_MoveX, x, 0);
		return GUI_Message_MoveX;
	}
	else if(my == GUI_Message_MoveY && mx == GUI_Message_Default)
	{
		GetGUI()->PostMessage(this, NULL, GUI_Message_MoveY, 0, y);
		return GUI_Message_MoveY;
	}

	return 0;
}

void CGUIElement::Draw()
{
	if(!Visible())
		return;

	int left = m_Rect.left;
	int right = m_Rect.right;
	int top = m_Rect.top;
	int bottom = m_Rect.bottom;

	if(m_pTexture == NULL)
	{
		glDisable(GL_TEXTURE_2D);

		if(m_pMaterial !=NULL)
		{
			glDisable(GL_LIGHTING);
			glColor4fv(m_pMaterial->GetAmbient().RGBA);
			// m_pMaterial->Bind();
		}
		else
		{
			glDisable(GL_LIGHTING);
			glColor3d(255, 255, 255);
		}

		glBegin(GL_QUADS);
		glVertex3d(left, bottom, 0);
		glVertex3d(right, bottom, 0);
		glVertex3d(right, top, 0);
		glVertex3d(left, top, 0);
		glEnd();
	}
	else
	{
		m_pTexture->Bind();

		if(m_pMaterial !=NULL)
		{
			glDisable(GL_LIGHTING);
			glColor3f(m_pMaterial->GetAmbient().R / 255.0f, m_pMaterial->GetAmbient().G / 255.0f, m_pMaterial->GetAmbient().B / 255.0f);
			// m_pMaterial->Bind();
		}
		else
			glColor3d(1, 1, 1);


		if(m_pTexture->IsMasked())
		{
			glAlphaFunc(GL_GREATER, 0.0f);
			glEnable(GL_ALPHA_TEST);
		}

		glBegin(GL_QUADS);
		//				Bottom Left							Bottom Left
		glTexCoord2f(m_TexCoord[0].x, m_TexCoord[0].y); glVertex3d(left, bottom, 0);
		//				Bottom Right is Bottom Left			Bottom Right
		glTexCoord2f(m_TexCoord[1].x, m_TexCoord[1].y); glVertex3d(right, bottom, 0);
		//				Top Right is Bottom Right			Top Right
		glTexCoord2f(m_TexCoord[2].x, m_TexCoord[2].y); glVertex3d(right, top, 0);
		//				Top Left is Top Right				Top Left
		glTexCoord2f(m_TexCoord[3].x, m_TexCoord[3].y); glVertex3d(left, top, 0);
		glEnd();

		if(m_pTexture->IsMasked())
			glDisable(GL_ALPHA_TEST);
	}

	CGUIElement *Iter = begin();
	set_ptr(Iter);

	while(Iter !=NULL)
	{
		Iter->Draw();
		Iter = next();
	}
}

int CGUIElement::OnKeyUp(UINT uiKey)
{

	return 1;
}

int CGUIElement::OnKeyDown(UINT uiKey)
{

	return 1;
}

int CGUIElement::Resize(tRect& NewRect)
{
	int ResizeX = 0, ResizeY = 0;
	if((NewRect.right - NewRect.left) >= GUI_MINIMUM_WIDTH)
	{
		if((NewRect.top - NewRect.bottom) >= GUI_MINIMUM_HEIGHT)
		{
			if(!IsFlagSet(GUI_Flag_RestrictSizeX))
			{
				m_Rect.top = NewRect.top;
				m_Rect.bottom = NewRect.bottom;
				ResizeY = GUI_Message_SizeY;
			}
		}

		if(!IsFlagSet(GUI_Flag_RestrictSizeY))
		{
			m_Rect.left = NewRect.left;
			m_Rect.right = NewRect.right;
			ResizeX = GUI_Message_SizeX;
		}
	}



	if(GetType() == GUI)
		return 1;

	if(ResizeX && ResizeY)
	{
		GetGUI()->PostMessage(this, GetChild(GUI_Border), GUI_Message_SizeXY);
		return GUI_Message_SizeXY;
	}

	if(ResizeX == GUI_Message_SizeX && ResizeY == 0)
	{
		GetGUI()->PostMessage(this, GetChild(GUI_Border), GUI_Message_SizeX);
		return GUI_Message_SizeX;
	}

	else if(ResizeY == GUI_Message_SizeY && ResizeX == 0)
	{
		GetGUI()->PostMessage(this, GetChild(GUI_Border), GUI_Message_SizeY);
		return GUI_Message_SizeY;
	}

	return 0;
}

void CGUIElement::SetType(eGUIControlType eWidgetType)
{
	m_eControlType = eWidgetType;
}

eGUIControlType& CGUIElement::GetType()
{
	return m_eControlType;
}

CGUI *CGUIElement::GetGUI()
{
	if(GetParent() !=NULL)
	{
		if(GetParent()->GetType() == GUI)
			return (CGUI *)GetParent();
		else
			return GetParent()->GetGUI();
	}

	return NULL;
}

void CGUIElement::SetGUI(CGUI *Root)
{
	m_pGUI = Root;
}

void CGUIElement::SetParent(CGUIElement *Parent)
{
	m_pParent = Parent;
}

CGUIElement *CGUIElement::GetParent()
{
	return m_pParent;
}

void CGUIElement::ProcessMessage(tGUIMessage& Message)
{

}

void CGUIElement::ProcessMessages()
{
	tGUIMessage *pMessage = NULL;

	for (UINT i=0; i<GetGUI()->GetMessageCount(); i++)
	{
		pMessage = GetGUI()->GetMessage(i);
		if(pMessage->uiMessage !=-1)
			ProcessMessage(*pMessage);
	}

	CGUIElement *pIterElement = begin();
	set_ptr(pIterElement);

	while(pIterElement !=NULL)
	{
		pIterElement->ProcessMessages();
		pIterElement = next();
	}
}

tRect& CGUIElement::GetRect()
{
	return m_Rect;
}

UINT CGUIElement::GetHeight()
{
	return (m_Rect.top - m_Rect.bottom);
}

UINT CGUIElement::GetWidth()
{
	return (m_Rect.right - m_Rect.left);
}

void CGUIElement::SetHeight(UINT Height)
{
	m_Rect.bottom = m_Rect.top - Height;
}

void CGUIElement::SetTexCoord(UINT uiIndex, float U, float V)
{
	if(uiIndex >=0 && uiIndex < 4)
	{
		m_TexCoord[uiIndex].x = U;
		m_TexCoord[uiIndex].y = V;
	}
}

tVERTEX2f& CGUIElement::GetTexCoord(UINT uiIndex)
{
	if(uiIndex >=0 && uiIndex < 4)
		return m_TexCoord[uiIndex];

	return tVERTEX2f(-1, -1);
}

CGUIElement *CGUIElement::GetChild(eGUIControlType eWidgetType)
{
	CGUIElement *pIterElement = begin();
	set_ptr(pIterElement);

	while(pIterElement !=NULL)
	{
		if(pIterElement->GetType() == eWidgetType)
			return pIterElement;

		pIterElement = next();
	}

	return NULL;
}

CGUIElement *CGUIElement::GetChild(UINT uiIndex)
{
	if(uiIndex >=0 && uiIndex < GetChildCount())
		return at(uiIndex);

	return NULL;
}

UINT CGUIElement::GetChildCount()
{
	return size();
}

CTexture *CGUIElement::GetTexture()
{
	return m_pTexture;
}

void CGUIElement::SetTexture(CTexture *pTexture)
{
	m_pTexture = pTexture;
}

CMaterial *CGUIElement::GetMaterial()
{
	return m_pMaterial;
}

void CGUIElement::SetMaterial(CMaterial *pMaterial)
{
	m_pMaterial = pMaterial;
}

void CGUIElement::HideSiblings(int iDeep)
{
	CGUIElement *pIterElement = begin();
	set_ptr(pIterElement);

	while(pIterElement !=NULL)
	{
		pIterElement->Hide();

		if(iDeep > 0)
			pIterElement->HideSiblings(iDeep - 1);
		else if(iDeep == -1)
			pIterElement->HideSiblings(iDeep);
		else if(iDeep == 0)
			return;

		pIterElement = next();
	}
}

void CGUIElement::ShowSiblings(int iDeep)
{
	CGUIElement *pIterElement = begin();
	set_ptr(pIterElement);

	while(pIterElement !=NULL)
	{
		if(iDeep > 0)
		{
			pIterElement->Show();
			pIterElement->ShowSiblings(iDeep - 1);
		}
		else if(iDeep == -1)
		{
			pIterElement->Show();
			pIterElement->ShowSiblings(iDeep);
		}
		else if(iDeep == 0)
			return;

		pIterElement = next();
	}
}

void CGUIElement::Show()
{
	m_bVisible = true;
}

void CGUIElement::Hide()
{
	m_bVisible = false;
}

bool CGUIElement::Visible()
{
	return m_bVisible;
}

void CGUIElement::SetFlag(eGUIFlag eFlag, bool bSet)
{
	if(bSet == true)
		m_dwFlags|= eFlag;
	else
		m_dwFlags^= eFlag;
}

bool CGUIElement::IsFlagSet(eGUIFlag eFlag)
{
	return (m_dwFlags & eFlag);
}

bool CGUIElement::IsChild(CGUIElement *pElement)
{
	CGUIElement *pIterElement = begin();
	set_ptr(pIterElement);

	while(pIterElement !=NULL)
	{
		if(pIterElement == pElement)
			return true;

		pIterElement = next();
	}

	return false;
}

// Retrieves/Moves element in parent' list of elements up to specified order index
void CGUIElement::SetZOrder(UINT uiZOrder)
{
	CGUIElement *pParent = GetParent();
	if(pParent !=NULL)
	{
		if(uiZOrder >=0 && uiZOrder < pParent->size())
			pParent->move_to(uiZOrder, this);
	}
}

unsigned int CGUIElement::GetZOrder()
{
	CGUIElement *parent = GetParent();
	if(parent == 0)
		return 0;
	else
	{
		for (UINT i=0; i<parent->GetChildCount(); i++)
		{
			if(parent->GetChild(i) == this)
				return i;
		}
	}
	return 1;
}

void CGUIElement::SetRect(tRect& NewRect)
{
	m_Rect = NewRect;
}

void CGUIElement::RemoveChild(CGUIElement *pChild)
{
	if(pChild == NULL)
		return;

	CGUIElement *pIterElement = begin();
	set_ptr(pIterElement);

	while(pIterElement !=NULL)
	{
		if(pIterElement == pChild)
		{
			delete remove(pChild);
			return;
		}

		pIterElement = next();
	}
}

bool CGUIElement::IsAutoCalc()
{
	return m_bAutoCalc;
}

void CGUIElement::SetAutoCalc(bool bAutoCalc)
{
	m_bAutoCalc = bAutoCalc;
}

CString CGUIElement::GetElementType()
{
	switch(GetEntityType())
	{
		case Entity_GUIElement:
			return CString("Element");
		case Entity_GUI:
			return CString("GUI");
		case Entity_GUIBorder:
			return CString("Border");
		case Entity_GUIStatic:
			return CString("Static");
		case Entity_GUITextBox:
			return CString("TextBox");
		case Entity_GUIButton:
			return CString("Button");
		case Entity_GUIScrollBar:
			return CString("ScrollBar");
		case Entity_GUIProgressBar:
			return CString("ProgressBar");
		case Entity_GUIDropList:
			return CString("DropList");
		case Entity_GUITabControl:
			return CString("TabControl");
		case Entity_GUIWindow:
			return CString("Window");
		case Entity_GUIRadioGroup:
			return CString("RadioGroup");
		case Entity_GUIMenu:
			return CString("Menu");
		case Entity_GUIMenuEntry:
			return CString("MenuEntry");
		case Entity_GUIListBox:
			return CString("ListBox");
	}
	return CString("Unknown");
}

void CGUIElement::Destroy()
{
	erase();
}

bool CGUIElement::IsOfType(eEntityType eType)
{
	if(eType == Entity_GUIElement)
		return true;

	return CXMLResource::IsOfType(eType);
}
