#include "StdAfx.h"
#include "GUI.h"

CGUIEventHandler::CGUIEventHandler(UINT MaxMessages)
{
	m_uiMessageCount = MaxMessages;
	m_pMessages = new tGUIMessage[MaxMessages];
	m_uiLastUnusedMessage = 0;
}

CGUIEventHandler::~CGUIEventHandler()
{
	if(m_pMessages !=NULL)
		SAFEDEL_ARRAY(m_pMessages)
}

int CGUIEventHandler::PostMessage(CGUIElement *pSender, CGUIElement *pReceiver, eGUIMessage eMessage, DWORD lParam, DWORD wParam, UINT uiTimeoutMs)
{
	tGUIMessage *NewMessage = NULL;
	UINT NewUnusedMessage = 0;


	for (UINT i=0; i<m_uiMessageCount; i++)
	{
		if(m_pMessages[i].bUsed == false && NewMessage != NULL)
		{
			m_uiLastUnusedMessage = i;
			break;
		}

		if(m_pMessages[i].bUsed == false && NewMessage == NULL)
			NewMessage = (tGUIMessage *)&m_pMessages[i];
	}

	if(NewMessage !=NULL)
	{
		NewMessage->pReceiver = pReceiver;
		NewMessage->pSender = pSender;
		NewMessage->uiMessage = eMessage;
		NewMessage->dwTimeout = uiTimeoutMs;
		NewMessage->dwCreateTime = m_Timer.GetAbsoluteTime();
		NewMessage->dwParam[0] = lParam;
		NewMessage->dwParam[1] = wParam;
		NewMessage->bUsed = true;

		return 1;
	}

	return 0;
}

void CGUIEventHandler::Dispatch(bool bOnce)
{
	for (int i=0; i<m_uiMessageCount; i++)
	{
		tGUIMessage *pMessage = (tGUIMessage *)&m_pMessages[i];
		if(m_pMessages[i].bUsed)
		{
			double CurrentTime = m_Timer.GetAbsoluteTime();
			if(CurrentTime - m_pMessages[i].dwCreateTime)
			{
				if(pMessage->pReceiver !=NULL)
				{
					pMessage->pReceiver->ProcessMessage(*pMessage);
					if(bOnce)
						pMessage->bUsed = false;
				}
			}
		}		
	}
}

tGUIMessage *CGUIEventHandler::GetMessage(UINT uiIndex)
{
	if(uiIndex >=0 && uiIndex < m_uiMessageCount)
		return (tGUIMessage *)&m_pMessages[uiIndex];

	return NULL;
}

void CGUIEventHandler::Flush()
{
	for (int i=0; i<m_uiMessageCount; i++)
	{
		tGUIMessage *pMessage = (tGUIMessage *)&m_pMessages[i];
		pMessage->bUsed = false;
		pMessage->uiMessage = 0;
		pMessage->pSender = pMessage->pReceiver = NULL;
	}
}
