#include "StdAfx.h"
#include "GUI.h"

CGUIRadioGroup::CGUIRadioGroup()
{
	SetEntityType(Entity_GUIRadioGroup);
	Initialize();
}

CGUIRadioGroup::~CGUIRadioGroup()
{
	Destroy();
}

void CGUIRadioGroup::Initialize()
{
	SetType(GUI_RadioGroup);
	m_iCurBtnSel = -1;
}

int CGUIRadioGroup::Create(CGUIElement *pParent, tRect WidgetRect, CCustomFont *pFont, CTexture *pTexture, CMaterial *pMaterial, bool bBorder)
{
	return CGUIStatic::Create(pParent, WidgetRect, pFont, pTexture, pMaterial, bBorder);
}

int CGUIRadioGroup::LoadXML(TiXmlNode *pDataNode, CString strFilename)
{
	TiXmlElement* pXMLElement = NULL;
	const char *pcValue = NULL;

	int iRetValue = CGUIStatic::LoadXML(pDataNode, strFilename);	// LoadXML the data of derivative
	if(iRetValue !=1)
		return iRetValue;

	pXMLElement = m_pXMLElement;

	pcValue = pXMLElement->Attribute("CurSel");
	if(pcValue !=NULL)
	{
		SetCurSel(atoi(pcValue));
		pcValue = NULL;
	}
	else
	{
		CGlobalLogger::GetSingleton().Write("\t**********General Warning\t**********\n");
		CGlobalLogger::GetSingleton().Write("RadioGroup (Name: %s, ID: %d) doesn't have Current iSelection (CurSel) Attribute specified. Assuming default...", GetName(), GetID());
	}

	return iRetValue;	// All went OK, return 1
}

int CGUIRadioGroup::SaveXML(TiXmlNode *pDataNode, CString strFilename)
{
	// Nowhere to save!			Nothing to save!
	if(pDataNode == NULL && strFilename == "")
		return -1;

	char Buf[128] = "";
	TiXmlElement *pSaveXMLElement = NULL;

	if(pDataNode != NULL)
		pSaveXMLElement = pDataNode->ToElement();
	else
		pSaveXMLElement = new TiXmlElement("RadioGroup");

	pSaveXMLElement->SetAttribute("CurSel", m_iCurBtnSel);

	int iRetValue = CGUIElement::SaveXML(pSaveXMLElement, strFilename);

	if(pDataNode == NULL)
		SAFEDEL(pSaveXMLElement)

	return iRetValue;
}

void CGUIRadioGroup::Draw()
{
	if(!Visible())
		return;

	//Comment the next line if you want the rectangle of radio group to be drawn
	HideSiblings();
	CGUIStatic::Draw();
	ShowSiblings();

	CGUIElement *pIterElement = begin();
	set_ptr(pIterElement);

	while(pIterElement !=NULL)
	{
		if(pIterElement->GetType() == GUI_Button || pIterElement->GetType() == GUI_Border)
			pIterElement->Draw();

		pIterElement = next();
	}
}

void CGUIRadioGroup::SetCurSel(int uiIndex)
{
	if(uiIndex >= 0 && uiIndex < GetChildCount())
	{
		m_iCurBtnSel = uiIndex;

		CGUIElement *pIterElement = begin();
		set_ptr(pIterElement);

		UINT Count = 0;
		while(pIterElement !=NULL)
		{
			if(pIterElement->GetType() == GUI_Button)
			{
				if(Count == uiIndex)
					((CGUIButton *)pIterElement)->SetState(false, 1);
				else
					((CGUIButton *)pIterElement)->SetState(false, 0);

				Count++;
			}
			
			pIterElement = next();
		}
	}
	else
		m_iCurBtnSel = -1;
}

void CGUIRadioGroup::ProcessMessage(tGUIMessage& Message)
{
	switch(Message.uiMessage)
	{
	case GUI_Message_ButtonPressed:
		{
			UINT Count = 0;
			CGUIElement *pIterElement = begin();
			set_ptr(pIterElement);

			while(pIterElement !=NULL)
			{
				if(pIterElement->GetType() == GUI_Button)
				{
					if((pIterElement == Message.pSender))
					{
						SetCurSel(Count);
						return;
					}

					Count++;
				}
				
				pIterElement = next();
			}

			break;
		}
	}
}

int CGUIRadioGroup::GetCurSel()
{
	return m_iCurBtnSel;
}

bool CGUIRadioGroup::IsOfType(eEntityType eType)
{
	if(eType == Entity_GUIRadioGroup)
		return true;

	return CGUIStatic::IsOfType(eType);
}
