#ifndef INPUTDEVICESTATE_H
#define INPUTDEVICESTATE_H

#pragma once
#include "Entity.h"

#define KEY_DEFAULT 0x00000
#define KEY_UP		0xA0
#define KEY_DOWN	0x0A

//! Mouse Device class
/*!
	This class is an interface for the device state, either it is mouse, keyboard or any other.
*/
class CInputDeviceState
{
	CREATOR(CInputDevice);
protected:

	CInputDeviceState();
	virtual ~CInputDeviceState();

	UINT *m_uipKeys;						/*!< Pointer to a future array of keys for the device */
	UINT m_uiKeyCount;						/*!< Number of keys for the device */

	#ifdef DIRECT_INPUT
		LPDIRECTINPUTDEVICE8 m_pDIDevice;	/*!< Pointer to DI Device */
	#endif
	/*!
		Method details:
		Destroys the device
	*/
	virtual void Destroy();

public:
	/*!
		Method details:
		@param uiKeyIndex - Index of the key to query
		\return Key state (KEY_UP or KEY_DOWN)
	*/
	virtual UINT GetKeyState(UINT uiKeyIndex);
	/*!
		Method details:
		\return Total number of virtual keys
	*/
	virtual UINT GetKeyCount();
	/*!
		Method details:
		@param uiKey - Key index
		@param uiState - Key state (either KEY_DEFAULT, KEY_DOWN or KEY_UP)
	*/
    virtual void SetKeyState(UINT uiKey, UINT uiState);
	/*!
		Method details:
		@param uiKey - Index of the key to query
		\return True if key is down, false otherwise
	*/
	virtual bool IsKeyDown(UINT uiKey);
	/*!
		Method details:
		@param uiKey - Index of the key to query
		\return True if key has just been released, false otherwise
	*/
	virtual bool IsKeyUp(UINT uiKey);
	/*!
		Method details:
		\return Varied returns
	*/
	virtual UINT Poll() = 0;
};

#endif