#include "StdAfx.h"
#include "InputEngine.h"

CInputEngine::CInputEngine()
{
	m_pKeyboard = NULL;
	m_pMouse = NULL;
//	m_pGamepad = NULL;
//	m_pJoystick = NULL;

	CGlobalLogger::GetSingleton().Write("Input Engine - Instantiated\n");
}

CInputEngine::~CInputEngine()
{
	CGlobalLogger::GetSingleton().Write("Input Engine - Destroyed\n");
	Destroy(); 
}

int CInputEngine::Create()
{
	m_pMouse = new CMouse;
	m_pKeyboard = new CKeyboard;

	#ifdef DIRECT_INPUT
		HRESULT Result = DirectInput8Create(GetModuleHandle(NULL), DIRECTINPUT_VERSION, 
                                         IID_IDirectInput8, (VOID**)&g_pDI, NULL );
		if(FAILED(Result))
			return -1;
	#endif

	m_pMouse->Create();
	m_pKeyboard->Create();

	return 1;
}

int CInputEngine::Poll()
{
	if(m_pKeyboard !=NULL)
		m_pKeyboard->Poll();

	if(m_pMouse !=NULL)
		m_pMouse->Poll();

	return 1;
}

CMouse *CInputEngine::GetMouse()
{
	return m_pMouse;
}

CKeyboard *CInputEngine::GetKeyboard()
{
	return m_pKeyboard;
}

void CInputEngine::Destroy()
{
	SAFEDEL(m_pKeyboard)
	SAFEDEL(m_pMouse)
//	SAFEDEL(m_pGamepad)
//	SAFEDEL(m_pJoystick)

	#ifdef DIRECT_INPUT
		if(g_pDI !=NULL)
		{
			g_pDI->Release();
			g_pDI = NULL;
		}
	#endif
}

bool CInputEngine::IsOfType(eEntityType eType)
{
	if(eType == Entity_InputEngine)
		return true;

	return CEntity::IsOfType(eType);
}
