#ifndef INPUTENGINE_H
#define INPUTENGINE_H

#pragma once
#include "Mouse.h"
#include "Keyboard.h"
// #include "Gamepad.h"
// #include "Joystick.h"

//! Input Engine class
/*!
	This class manages all the input from hardware-based devices (keyboard, mouse, gamepad etc)
	and provides easy-to-use interface to access them/their values/keys
*/
class CInputEngine : public CEntity, public CSingleton<CInputEngine>
{
public:
	CInputEngine();
	virtual ~CInputEngine();

public:

	/*!
		Method details:
		\return Pointer to mouse device
	*/
	CMouse *GetMouse();
	/*!
		Method details:
		\return Pointer to keyboard device
	*/
	CKeyboard *GetKeyboard();
	/*!
		Method details:
		\return 1 if engine has been created succesfully, 0 otherwise
	*/
	int Create();
	/*!
		Method details:
		\return 1 if engine has polled something new, 0 otherwise
	*/
	int Poll();
	/*!
		Method details:
		Destroys the engine
	*/
	void Destroy();
	/*!
		Method details:
		This is an implementation of Custom RTTI
	*/
	virtual bool IsOfType(eEntityType eType);

protected:

	CKeyboard	*m_pKeyboard;			/*!< Pointer to keyboard device class */
	CMouse		*m_pMouse;				/*!< Pointer to mouse device class */
//	CJoystick	*m_pJoystick;			/*!< Pointer to joystick device class */
// 	CGamepad	*m_pGamepad;			/*!< Pointer to gamepad device class */
};

#endif