#ifndef KEYBOARD_H
#define KEYBOARD_H

#pragma once
#include "InputDevice.h"
#include "InputDeviceState.h"
#include "KeyTable.h"

//! Mouse Device class
/*!
	This class provides an easy and clear shell for a keyboard device
*/
class CKeyboard : public CInputDevice, public CInputDeviceState
{
	CREATOR(CInputEngine);

protected:

	CKeyboard();
	virtual ~CKeyboard();

	/*!
		Method details:
		\return 1 if device has been created succesfully, 0 otherwise
	*/
	virtual int Create();
	/*!
		Method details:
		Destroys the device
	*/
	virtual void Destroy();
	/*!
		Method details:
		\return Varied return values
	*/
	virtual UINT Poll();

public:
	/*!
		Method details:
		@param cKey - Index of the key to query
		\return True if key is down, false otherwise
	*/
	bool IsKeyDown(char cKey);
	/*!
		Method details:
		@param uiButtonIndex - Index of the key to query
		\return True if key has just been released, false otherwise
	*/
	bool IsKeyUp(char cKey);
	/*!
		Method details:
		@param cKey - Index of the key to query
		\return Key state (KEY_UP or KEY_DOWN)
	*/
	UINT GetKeyState(UINT uiKeyIndex);
	/*!
		Method details:
		This is an implementation of Custom RTTI
	*/
	virtual bool IsOfType(eEntityType eType);
};

#endif