// Logger.h: interface for the CLogger class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_LOGGER_H__D5C75D23_D1A5_43BE_A6F3_3778FDD8A669__INCLUDED_)
#define AFX_LOGGER_H__D5C75D23_D1A5_43BE_A6F3_3778FDD8A669__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// Define this if you want to use global logger
#define USE_GLOBAL_LOGGER 1

#include "Singleton.h"
#include <stdarg.h>
#include <stdio.h>

//! Logger class. 
/*!
	This class logs passed messages (that can be formatted) into the\n
	specified file. It has the ability to overwrite, bAppend, and keep the\n
	specified file open until the explicit command to close is given.
*/
class CLogger
{
public:

	CLogger();
	CLogger(char *pcFilename, bool bKeepOpen = false, bool bAppend = false);
	virtual ~CLogger();
	
    /*!
		Opens file for write access\n
		strFilename - The path and file name of the file\n
		bKeepOpen - keep the file open until the Close() is called\n
		bAppend - bAppend to the end of file
    */
	virtual int Open(char *pcFilename, bool bKeepOpen = false, bool bAppend = false);
	
	
    /*!
		Writes CString in format specified to opened file\n
		format - as in printf, sprintf, and the rest (uses va_start, va_end)
    */
	virtual int Write(const char *pcFormat, ...);
	
    /*!
		Appends specified CString in given format to opened file\n
		format - as in printf, sprintf, and the rest (uses va_start, va_end)
    */
	virtual int Append(const char *pcFormat, ...);

	/*!
		Erases opened file
    */
	virtual int Erase();
	/*!
		Closes opened file
    */
	virtual int Close();

	/*! Returns an error code */
	int GetError();

private:
	
	FILE *m_pFile;				/*!< This variable stores a pointer to the opened file. */
	char m_strFilename[256];	/*!< This variable stores a file name. */
	bool m_bKeepOpen;			/*!< This variable indicates whether the file should be open until Close() is called or not. */
};
#endif // !defined(AFX_LOGGER_H__D5C75D23_D1A5_43BE_A6F3_3778FDD8A669__INCLUDED_)

#ifdef USE_GLOBAL_LOGGER
//! Global Logger class. 
/*!
	This class logs passed messages (that can be formatted) into the\n
	globally specified file that is open until the Close() is called, or\n
	CGlobalLogger is destroyed.
*/
class CGlobalLogger : public CLogger, public CSingleton<CGlobalLogger>
{
public:

	CGlobalLogger();
	CGlobalLogger(char *pcFilename);
	virtual ~CGlobalLogger();

	void SetLogLevel(int iLogLevel);
	int GetLogLevel();
	/*!
      This function opens the given file (path and file name) and keeps it open
    */
	virtual int Open(char *pcFilename, bool bKeepOpen = true);
	/*!
      This function closes the file.
    */
	virtual int Close();

private:
	int m_iLogLevel;

};

#endif