#ifndef MATERIALMANAGER_H
#define MATERIALMANAGER_H

#pragma once
#include "ResourceManager.h"
#include "OpenGLMaterial.h"

//!Material Manager class
/*!
	This class provides means to managing various materials (either DirectX or OpenGL ones)
	It manages a collection of materials (although not pooled yet!) and allow to retrieve
	them by index, name, ID or filename.
*/
class CMaterialManager : public CResourceManager, public CSingleton<CMaterialManager>
{

public:
	CMaterialManager();
	virtual ~CMaterialManager();

	/*!
		Method details:
		@param pDataNode - Pointer to XML Data node containing data for the material
		@param strFilename - Name of the file containing data for the material
		@param uiID - ID for the material to be loaded
		@param strName - Name for the material to be loaded
		\return  Pointer to the material if it has been loaded succesfully, NULL if not
	*/
	virtual CMaterial *LoadResource(TiXmlNode *pDataNode, CString strFilename, UINT uiID = 0, CString strName = "");
	/*!
		Method details:
		@param uiIndex - Index by which to retrieve the material
		@param uiID - ID by which to retrieve the material
		@param strName - Name by which to retrieve the material
		@param strFilename - Filename by which to retrieve the material
		\return Pointer to the material if it has been found using given parameters, NULL if not
	*/
	virtual CMaterial *GetResource(int uiIndex, int uiID = -1, CString strName = "", CString strFilename = "");
	/*!
		Method details:
		@param strName - Name of the material to add
		@param uiID - ID of the material to add
		@param ResourceType - Type of the material to add
		\return Pointer to the material if it has been added succesfully using given parameters, NULL if not
	*/
	virtual CMaterial *AddResource(CString strName, UINT uiID = 0, eEntityType ResourceType = Entity_Default);
	/*!
		Method details:
		@param pResource - Pointer to the material to remove
		\return 1 if it has been found and removed succesfully, 0 otherwise
	*/
	int RemoveResource(CMaterial *pMaterial);

	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that holds data for this texture
		@param strFilename - String containing filename where the data for the material is stored
	*/
	virtual int LoadXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that will be the parent hold the data for this texture
		@param strFilename - String containing filename where the data for the material is stored
	*/
	virtual int SaveXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		This is where clean-up occurs
	*/
	virtual void Destroy();
	/*!
		Method details:
		This is an implementation of Custom RTTI
	*/
	virtual bool IsOfType(eEntityType eType);
};

#endif