#ifndef MOUSE_H
#define MOUSE_H

#pragma once
#include "InputDevice.h"
#include "InputDeviceState.h"

//! Mouse Device class
/*!
	This class provides an easy and clear shell for ANY mouse device
*/
class CMouse : public CInputDevice, public CInputDeviceState
{
	CREATOR(CInputEngine);

protected:
	CMouse();
	virtual ~CMouse();

	/*!
		Method details:
		\return 1 if device has been created succesfully, 0 otherwise
	*/
	virtual int Create();
	/*!
		Method details:
		Destroys the device
	*/
	virtual void Destroy();
	/*!
		Method details:
		\return Varied return values
	*/
	virtual UINT Poll();

public:
	/*!
		Method details:
		@param uiButtonIndex - Index of the button to query
		\return True if button is down, false otherwise
	*/
	bool IsButtonDown(UINT uiButtonIndex);
	/*!
		Method details:
		@param uiButtonIndex - Index of the button to query
		\return True if button has just been released, false otherwise
	*/
	bool IsButtonUp(UINT uiButtonIndex);
	/*!
		Method details:
		@param uiKeyIndex - Index of the key to query
		\return Key state (KEY_UP or KEY_DOWN)
	*/
	UINT GetKeyState(UINT uiKeyIndex);
	/*!
		Method details:
		\return Const. Reference to 2D coordinates of the mouse cursor
	*/
	tVERTEX2d& GetCursorPos();
	/*!
		Method details:
		\return Const. Reference to previous 2D coordinates of the mouse cursor
	*/
	tVERTEX2d& GetPrevCursorPos();
	/*!
		Method details:
		\@param x,y - New (updated) coordinates for the mouse
	*/
	void SetCursorPos(int x, int y);
	/*!
		Method details:
		This is an implementation of Custom RTTI
	*/
	virtual bool IsOfType(eEntityType eType);

protected:
	tVERTEX2d m_CursorPos;				/*!< Mouse cursor position */
	tVERTEX2d m_PrevCursorPos;			/*!< Previous mouse cursor position */

	tVERTEX2d m_CursorSpeed;			/*!< Mouse cursor speed (dx & dy) */

	#ifdef DIRECT_INPUT
		DIMOUSESTATE2 m_MouseState;		/*!< Mouse state for use with DI */
	#endif
};

#endif
