#ifndef OPENGL_MATERIAL_H
#define OPENGL_MATERIAL_H

#pragma once
#include "material.h"

//! OpenGL Specific Material class
/*!
	This class provides means for 4 different material colors
	as well as loading/saving it in XML for use with OpenGL.
*/
class COpenGLMaterial : public CMaterial
{
	CREATOR(CMaterialManager);

protected:

	COpenGLMaterial();
	virtual ~COpenGLMaterial();

	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that holds data for the material
		@param strFilename - String containing filename where the data for the material is stored
	*/
	virtual int LoadXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that will be the parent hold the data for this material
		@param strFilename - String containing filename where the data for the material is stored
	*/
	virtual int SaveXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		Binds this material as current one
	*/
	virtual void Bind();
	/*!
		Method details:
		Clean-up
	*/
	void Destroy();
	/*!
		Method details:
		This is an implementation of Custom RTTI
	*/
	virtual bool IsOfType(eEntityType eType);
};

#endif