// OpenGLTexture.h: interface for the COpenGLTexture class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_OPENGLTEXTURE_H__8DF78D3D_F090_4E13_93BB_8906C272AD09__INCLUDED_)
#define AFX_OPENGLTEXTURE_H__8DF78D3D_F090_4E13_93BB_8906C272AD09__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Texture.h"
//! OpenGL Specific Texture class
/*!
	This class is a texture-derived one that provides
	itself for use ONLY with OpenGL.
*/
class COpenGLTexture : public CTexture  
{
	CREATOR(CTextureManager);
protected:
	  
    COpenGLTexture();
	virtual ~COpenGLTexture();

	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that holds data for the texture
		@param strFilename - String containing filename where the data for the texture is stored
	*/
	virtual int LoadXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that will be the parent hold the data for this texture
		@param strFilename - String containing filename where the data for the texture is stored
	*/
	virtual int SaveXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		Clean-up
	*/
	void Destroy();
	/*!
		Method details:
		Releases the texture
	*/
	virtual void Release();
	/*!
		Method details:
		Sets this texture as a current one
	*/
	void Bind();
	/*!
		Method details:
		\return 1 if created succesfully, 0 otherwise
	*/
	UINT Create();
	/*!
		Method details:
		This is an implementation of Custom RTTI
	*/
	virtual bool IsOfType(eEntityType eType);

private:
	//----------------------------------------------------------------------
	// OpenGL related members
	//--
	UINT m_uiGLTextureID;		/*!< OpenGL Texture ID */
	UINT m_uiGLFormat;			/*!< OpenGL Texture Format */
	UINT m_uiGLInternalFormat;	/*!< OpenGL Internal Format */
	UINT m_uiPixelStride;		/*!< OpenGL Pixel Stride */

};

bool CheckSize(UINT Size);

#endif // !defined(AFX_OPENGLTEXTURE_H__8DF78D3D_F090_4E13_93BB_8906C272AD09__INCLUDED_)
