#ifndef RESOURCEMANAGER_H
#define RESOURCEMANAGER_H

#pragma once

#include "XMLResource.h"
#include "DoubleList.h"

//! Basic Resource Manager class
/*!
	This class serves as a base class for other resource managers.
	It manages a collection of resources (although not pooled yet!) and allow to retrieve
	them by index, name, ID or filename.
*/
class CResourceManager : public CDoubleList<CXMLResource *>, public CXMLResource
{
public:
	CResourceManager();
	virtual ~CResourceManager();

public:
	/*!
		Method details:
		@param pDataNode - Pointer to XML Data node containing data for the resource
		@param strFilename - Name of the file containing data for the resource
		@param uiID - ID for the resource to be loaded
		@param strName - Name for the resource to be loaded
		\return  Pointer to the resource if it has been loaded succesfully, NULL if not
	*/
	virtual CXMLResource *LoadResource(TiXmlNode *pDataNode, CString strFilename, UINT uiID = 0, CString strName = "") = 0;
	/*!
		Method details:
		@param uiIndex - Index by which to retrieve the resource
		@param uiID - ID by which to retrieve the resource
		@param strName - Name by which to retrieve the resource
		@param strFilename - Filename by which to retrieve the resource
		\return Pointer to the resource if it has been found using given parameters, NULL if not
	*/
	virtual CXMLResource *GetResource(int uiIndex, int uiID = -1, CString strName = "", CString strFilename = "");
	/*!
		Method details:
		@param strName - Name of the resource to add
		@param uiID - ID of the resource to add
		@param ResourceType - Type of the resource to add
		\return Pointer to the resource if it has been added succesfully using given parameters, NULL if not
	*/
	virtual CXMLResource *AddResource(CString strName, UINT uiID = 0, eEntityType ResourceType = Entity_Default);
	/*!
		Method details:
		@param pResource - Pointer to the resource to remove
		\return 1 if it has been found and removed succesfully, 0 otherwise
	*/
	int RemoveResource(CXMLResource *pResource);

	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that holds data for this resource
		@param strFilename - String containing filename where the data for the resource is stored
	*/
	virtual int LoadXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that will be the parent hold the data for this resource
		@param strFilename - String containing filename where the data for the resource is stored
	*/
	virtual int SaveXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		This is where clean-up occurs
	*/
	virtual void Destroy();
	/*!
		Method details:
		This is an implementation of Custom RTTI
	*/
	virtual bool IsOfType(eEntityType eType);
};

#endif