#ifndef TEXTUREFRAMEANIMATION_H
#define TEXTUREFRAMEANIMATION_H

#pragma once

#include "TextureManager.h"
#include "FrameAnimation.h"
//! Texture Frame Animation class
/*!
	This class is an implementation of the CFrameAnimation class
	that uses textures to provide means of animated texture(s).
*/
class CTextureFrameAnimation : public CFrameAnimation<tRect>
{
public:
	CTextureFrameAnimation();
	virtual ~CTextureFrameAnimation();

public:
	int Create(UINT FrameCount, UINT DataCount);

	/*!
		Method details:
		@param fTimePerFrame - Amount of time for the frame to wait before advancing to next
		\return 1 If animation is complete, 0 otherwise
	*/
	virtual int Animate(double fTimePerFrame);
	/*!
		Method details:
		@param fPauseTime - Pause for infinite amount of time or one specified by user
		\return 1 If animation is paused, 0 otherwise
	*/
	virtual int Pause(double fPauseTime = INFINITE);
	/*!
		Method details:
		@param fTimeToWait - Time to wait before resuming the animation
		\return 1 If it is still waiting, 0 otherwise
	*/
	virtual int Resume(double fTimeToWait = 0);
	/*!
		Method details:
		\return 1 If animation has been stopped, 0 otherwise
	*/
	virtual int Stop();
	/*!
		Method details:
		Clean-up method
	*/
	virtual void Destroy();
	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that holds data for the animation
		@param strFilename - String containing filename where the data for the animation is stored
	*/
	virtual int LoadXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that will be the parent hold the data for this animation
		@param strFilename - String containing filename where the data for the animation is stored
	*/
	virtual int SaveXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		@param uiFrame - Index of the frame to set texture for
		@param uiTexIndex - Index of data that will contain pointer to the texture
		@param pTexture - Pointer to texture in texture manager
	*/
	void SetTexture(UINT uiFrame, UINT uiTexIndex, CTexture *pTexture = NULL);
	/*!
		Method details:
		@param uiFrameIndex - Index of the frame
		@param Rect - New frame rectangle
	*/
	void SetFrameRect(UINT uiFrameIndex, tRect& Rect);
	/*!
		Method details:
		\return Pointer to texture for current frame
	*/
	CTexture *GetTexture();
	/*!
		Method details:
		This is an implementation of Custom RTTI
	*/
	virtual bool IsOfType(eEntityType eType);

protected:
	CTexture **m_pTextures;					/*!< An array of pointers to textures */
	/*!
		Method details:
		@param pDataNode - Pointer to animation XML node
		@param uiIndex - index of the frame to save
		\return 1 if saved succesfully, 0 otherwise
	*/
	int SaveFrame(TiXmlNode *pDataNode, UINT uiIndex);
};

#endif