#include "StdAfx.h"
#include "TextureManager.h"

CTextureManager::CTextureManager()
{
	SetEntityType(Entity_TextureManager);
	CGlobalLogger::GetSingleton().Write("Texture Manager - Instantiated\n");
}

CTextureManager::~CTextureManager()
{
	Destroy();
	CGlobalLogger::GetSingleton().Write("Texture Manager - Destroyed\n");
}

CTexture *CTextureManager::AddResource(CString strName, UINT uiID, eEntityType ResourceType)
{
	switch(ResourceType)
	{
	case Entity_Default:
	case Entity_Texture:
	case Entity_OpenGLTexture:
		CTexture *pNewTexture = new COpenGLTexture;

		pNewTexture->SetID(uiID);
		pNewTexture->SetName(strName);

		push_back(pNewTexture);

		return pNewTexture;
		break;
	}

	return NULL;
}

int CTextureManager::LoadXML(TiXmlNode *pDataNode, CString strFilename)
{
	TiXmlElement* pXMLElement = NULL;
	const char *pcValue = NULL;

	int iRetValue = CResourceManager::LoadXML(pDataNode, strFilename);	// LoadXML the data of derivative
	if(iRetValue !=1)
		return iRetValue;

	pXMLElement = m_pXMLElement;

	if(stricmp(pXMLElement->Value(), "Texture") == 0)
	{
		CXMLResource *AddedResource = LoadResource(pDataNode, strFilename);
		if(AddedResource !=NULL)
		{
			CGlobalLogger::GetSingleton().Write("Texture (Name: %s, ID: %d) has been added successfully.\n", AddedResource->GetName().GetBuffer(), AddedResource->GetID()); 
			return 1;
		}
		else
			return 0;
	}
	// LoadXML Textures
	UINT TextureLoadedCount = 0;
	TiXmlNode *pIterXMLNode = pXMLElement->FirstChild();
	while(pIterXMLNode !=NULL)
	{
		if(stricmp(pIterXMLNode->Value(), "Texture") == 0)	// We have a Texture
		{
			CXMLResource *AddedResource = LoadResource(pIterXMLNode, "");
			if(AddedResource !=NULL)
			{
				CGlobalLogger::GetSingleton().Write("Texture (Name: %s, ID: %d) has been added successfully.\n", AddedResource->GetName().GetBuffer(), AddedResource->GetID()); 
				TextureLoadedCount++;
			}
		}

		pIterXMLNode = pXMLElement->IterateChildren(pIterXMLNode);
	}

	return TextureLoadedCount;
}

int CTextureManager::SaveXML(TiXmlNode *pDataNode, CString strFilename)
{
	// Nowhere to save!			Nothing to save!
	if(pDataNode == NULL && strFilename == "")
		return -1;

	char Buf[128] = "";
	TiXmlElement *pSaveXMLElement = NULL;

	if(pDataNode != NULL)
		pSaveXMLElement = pDataNode->ToElement();
	else
		pSaveXMLElement = new TiXmlElement("TextureDatabase");

	CTexture *pTexture = (CTexture *)begin();
	set_ptr(pTexture);

	while(pTexture !=NULL)
	{
		TiXmlElement TextureXMLElement("Texture");
		if(pTexture->SaveXML(&TextureXMLElement, ""))
			pSaveXMLElement->InsertEndChild(TextureXMLElement);

		pTexture = (CTexture *)next();
	}

	if(pDataNode == NULL && strFilename !="")
	{
		TiXmlDocument SaveDoc;
		SaveDoc.InsertEndChild(*pSaveXMLElement);
		int iRetValue = SaveDoc.SaveFile(strFilename);
		SAFEDEL(pSaveXMLElement)

		return iRetValue;
	}

	return 1;
}

CTexture *CTextureManager::LoadResource(TiXmlNode *pDataNode, CString strFilename, UINT uiID, CString strName)
{
	if(pDataNode != NULL)
	{
		const char *pcValue = NULL;
		TiXmlElement *pXMLElement = pDataNode->ToElement();

		pcValue = pXMLElement->Attribute("TexByID");
		if(pcValue !=NULL)
		{
			CTexture *ExistingTexture = GetResource(-1, atoi(pcValue), "");
			if(ExistingTexture !=NULL)
				return ExistingTexture;
			else
				return NULL;
		}

		pcValue = pXMLElement->Attribute("TexByName");
		if(pcValue !=NULL)
		{
			CTexture *ExistingTexture = GetResource(-1, -1, pcValue);
			if(ExistingTexture !=NULL)
				return ExistingTexture;
			else
				return NULL;
		}
	}

	CTexture *NewTexture = NULL;

	#ifdef OPENGL
		NewTexture = new COpenGLTexture;
	#endif

	if(NewTexture->LoadXML(pDataNode, strFilename))
	{
		if(NewTexture->GetID() == 0)
			NewTexture->SetID(uiID);

		if(NewTexture->GetName() == "")
			NewTexture->SetName(strName);

		if(NewTexture->GetName() !="")
		{
			if(GetResource(-1, -1, NewTexture->GetName()) == NULL)
			{
				push_back(NewTexture);
				return NewTexture;
			}
			else
			{
				SAFEDEL(NewTexture)
					return NULL;
			}
		}
		else
		{
			push_back(NewTexture);
			return NewTexture;
		}
	}
	else
		SAFEDEL(NewTexture)


		return NULL;
}

CTexture *CTextureManager::GetResource(int uiIndex, int uiID, CString strName, CString strFilename)
{
	if(uiID !=-1 && uiIndex == -1 && strName == "" && strFilename == "")	// get by uiID
	{
		CXMLResource *ResourcePtr = begin();
		set_ptr(ResourcePtr);

		while(ResourcePtr !=NULL)
		{
			if(ResourcePtr->GetID() == uiID)
				return (CTexture *)ResourcePtr;

			ResourcePtr = next();
		}
	}
	else if(strName !="" && uiIndex == -1 && uiID == -1 && strFilename == "")	// get by name
	{
		CXMLResource *ResourcePtr = begin();
		set_ptr(ResourcePtr);

		while(ResourcePtr !=NULL)
		{
			if(stricmp(ResourcePtr->GetName().GetBuffer(), strName.GetBuffer()) == 0)
				return (CTexture *)ResourcePtr;

			ResourcePtr = next();
		}
	}
	else if(uiIndex !=-1 && uiID == -1 && strName == "" && strFilename == "")	// get by index
	{
		if(uiIndex >=0 && uiIndex < size())
			return (CTexture *)at(uiIndex);
	}
	else if(strFilename !="" && uiIndex ==-1 && uiID == -1 && strName == "")
	{
		CXMLResource *ResourcePtr = begin();
		set_ptr(ResourcePtr);

		while(ResourcePtr !=NULL)
		{
			if(stricmp(((CTexture *)ResourcePtr)->GetTextureFilename().GetBuffer(), strFilename.GetBuffer()) == 0)
				return (CTexture *)ResourcePtr;

			ResourcePtr = next();
		}
	}

	return NULL;
}

int CTextureManager::RemoveResource(CTexture *pTexture)
{

	return 0;
}

void CTextureManager::Destroy()
{
	CResourceManager::Destroy();
}

bool CTextureManager::IsOfType(eEntityType eType)
{
	if(eType == Entity_TextureManager)
		return true;

	return CResourceManager::IsOfType(eType);
}