#ifndef TEXTUREMANAGER_H
#define TEXTUREMANAGER_H

#pragma once
#include "ResourceManager.h"
#include "OpenGLTexture.h"

//!Texture Manager class
/*!
	This class provides means to managing various textures (either DirectX or OpenGL ones)
	It manages a collection of textures (although not pooled yet!) and allow to retrieve
	them by index, name, ID or filename.
*/
class CTextureManager :	public CResourceManager, public CSingleton<CTextureManager>
{
public:
	CTextureManager();
	virtual ~CTextureManager();
	/*!
		Method details:
		@param pDataNode - Pointer to XML Data node containing data for the texture
		@param strFilename - Name of the file containing data for the texture
		@param uiID - ID for the texture to be loaded
		@param strName - Name for the texture to be loaded
		\return  Pointer to the texture if it has been loaded succesfully, NULL if not
	*/
	virtual CTexture *LoadResource(TiXmlNode *pDataNode, CString strFilename, UINT uiID = 0, CString strName = "");
	/*!
		Method details:
		@param uiIndex - Index by which to retrieve the texture
		@param uiID - ID by which to retrieve the texture
		@param strName - Name by which to retrieve the texture
		@param strFilename - Filename by which to retrieve the texture
		\return Pointer to the texture if it has been found using given parameters, NULL if not
	*/
	virtual CTexture *GetResource(int uiIndex, int uiID = -1, CString strName = "", CString strFilename = "");
	/*!
		Method details:
		@param strName - Name of the texture to add
		@param uiID - ID of the texture to add
		@param ResourceType - Type of the texture to add
		\return Pointer to the texture if it has been added succesfully using given parameters, NULL if not
	*/
	virtual CTexture *AddResource(CString strName, UINT uiID = 0, eEntityType ResourceType = Entity_Default);
	/*!
		Method details:
		@param pResource - Pointer to the texture to remove
		\return 1 if it has been found and removed succesfully, 0 otherwise
	*/
	int RemoveResource(CTexture *pTexture);

	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that holds data for this texture
		@param strFilename - String containing filename where the data for the texture is stored
	*/
	virtual int LoadXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that will be the parent hold the data for this texture
		@param strFilename - String containing filename where the data for the texture is stored
	*/
	virtual int SaveXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		This is where clean-up occurs
	*/
	virtual void Destroy();
	/*!
		Method details:
		This is an implementation of Custom RTTI
	*/
	virtual bool IsOfType(eEntityType eType);
};

#endif