#ifndef XMLRESOURCE_H
#define XMLRESOURCE_H

#pragma once

#include "Resource.h"
#include "TinyXML.h"

#define ERROR_XML_GENERAL 1001
#define ERROR_XML_PARSING_FAILED 1002
#define ERROR_XML_

//! XML Resource class
/*!
	This class serves as a provider for resources that are loaded from XML file.
*/
class CXMLResource : public CResource
{
public:

	CXMLResource();
	virtual ~CXMLResource();

	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that holds data for this widget
		@param strFilename - String containing filename where the data for the widget is stored
	*/
	virtual int LoadXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		@param pDataNode - Pointer to XML Node that will be the parent hold the data for this widget
		@param strFilename - String containing filename where the data for the widget is stored
	*/
	virtual int SaveXML(TiXmlNode *pDataNode, CString strFilename);
	/*!
		Method details:
		Clean-up occurs here
	*/
	virtual void Destroy();
	/*!
		Method details:
		\return File name for this XML resource
	*/
	CString GetFilename();
	/*!
		Method details:
		This is an implementation of Custom RTTI
	*/
	virtual bool IsOfType(eEntityType eType);
	
protected:
	/*!
		Method details:
		@param strFilename - Sets filename for this XML resource
	*/
	void SetFilename(CString strFilename);

	TiXmlElement *m_pXMLElement;		/*!< Pointer to XML element holding data for this XML resource */
	CString m_strFilename;				/*!< Filename of this resource (external) */
};

#endif